/*
 * ActivityForm JS - creates indicator input blocks depending on the selected act.
 *
 * Available indicator kinds are provided as data-indicator-kinds attribute of the select with acts optios.
 * For each indicator kind are given "id", "type", "icon" and "options" attributes.
 *
 * Each act option has data attribute indicators with the information on the indicators for this act.
 * For each indicator are given "id", "kind_id", "title", "units", "current_value" and "goal_value" attributes.
 *
 * Current activity indicators are provided as data-current-indicators attribute for the indicators_input_row.
 * For each activity indicator are given "id", "indicator_id", and "indicator_value" attributes.
 * */
let ActivityForm = {
  form: null,
  indicatorsRow: null,
  indicatorKinds: {},
  currentActivityIndicators: {},

  init(formElement) {
    let aSelect = document.getElementById("activity_act_id");

    ActivityForm.form = formElement;

    ActivityForm.indicatorKinds = {}
    JSON.parse(aSelect.dataset.indicatorKinds).forEach(function(ik) {
      ActivityForm.indicatorKinds[ik.id] = ik
    });

    ActivityForm.indicatorsRow = document.getElementById("indicators_input_row");

    ActivityForm.currentActivityIndicators = {}
    JSON.parse(ActivityForm.indicatorsRow.dataset.currentActivityIndicators).forEach(function(ai) {
      ActivityForm.currentActivityIndicators[ai.indicator_id] = ai
    });


    ActivityForm.updateIndicatorBlock(aSelect.selectedOptions[0]);

    aSelect.addEventListener("change", ActivityForm.actionSelectedCallback)
  },

  actionSelectedCallback() {
    // NOTE: here this is the select element
    let selected = this.selectedOptions[0];

    ActivityForm.updateIndicatorBlock(selected)
  },

  updateIndicatorBlock(selected) {
    if (typeof(selected.dataset.indicators) != "undefined") {
      let indicators = JSON.parse(selected.dataset.indicators);

      ActivityForm.indicatorsRow.innerHTML = "";

      indicators.forEach(function(indicator, index) {
        ActivityForm.indicatorsRow.append(ActivityForm.createIndicatorBlock(indicator, index));
      })
    } else {
      ActivityForm.indicatorsRow.innerHTML = "";
    }
  },


  createIndicatorBlock(indicator, n) {
    let kind = ActivityForm.indicatorKinds[indicator.kind_id];

    let iBlock = document.createElement("div");
    iBlock.classList.add("col")
    iBlock.classList.add("mb-4")
    iBlock.classList.add("ef-indicator-input-row")
    iBlock.classList.add("ef-indicator-"+kind["type"]+"-input-row")

    let container = document.createElement("div")
    container.classList.add("card")
    container.classList.add("py-2")

    let cardHeader = ActivityForm._createCardHeader(kind, indicator)

    let cardBody = document.createElement("div");
    cardBody.classList.add("card-body", "mt-2");

    let inputRow = document.createElement("div");
    inputRow.classList.add("row", "no-gutters");

    inputRow.appendChild(ActivityForm._createIndicatorCol(indicator, kind, n));

    cardBody.appendChild(inputRow);

    container.appendChild(cardHeader);
    container.appendChild(cardBody);

    iBlock.appendChild(container);
    return iBlock;
  },

  _createIndicatorCol(indicator, kind, n) {
    let inputCol = document.createElement("div");
    inputCol.classList.add("col", "mr-2", "ef-indicator-input-col");

    let inputFormElement = document.createElement("div");
    inputFormElement.classList.add("form-group");

    // let inputLabel = document.createElement("label")
    // inputLabel.innerHTML = indicator["title"]
    // //"current_value="+indicator["current_value"]+", goal: "+indicator["goal_value"]+", kind_id="+indicator.kind_id+"kind_type="+kind["type"];

    let valueInput;

    if (kind["type"] == "phases") {
      valueInput = document.createElement("select");
      kind["options"].forEach(function(elem, i) {
        let opt = document.createElement("option");
        opt.text = elem;
        opt.value = i;
        valueInput.add(opt, null);
      })
    } else {
      valueInput = document.createElement("input");
      valueInput.setAttribute("type", "number");
      valueInput.setAttribute("step", "0.01"); // numeric and percentage indicator can have decimal values
    }

    valueInput.setAttribute("id", "activity_activity_indicators_"+n+"_indicator_value");
    valueInput.setAttribute("name", "activity[activity_indicators]["+n+"][indicator_value]");
    valueInput.classList.add("form-control");

    let idInput = document.createElement("input");
    idInput.setAttribute("type", "hidden");

    let currentActivityIndicator = ActivityForm.currentActivityIndicators[indicator.id];

    if ((typeof(currentActivityIndicator) === "undefined") || (currentActivityIndicator["id"] === null)) {
      // new activity indicator will be created, hense we provide indicator_id
      idInput.setAttribute("id", "activity_activity_indicators_"+n+"_indicator_id");
      idInput.setAttribute("name", "activity[activity_indicators]["+n+"][indicator_id]");
      idInput.setAttribute("value", indicator.id);

      valueInput.selectedIndex = parseInt(indicator.current_value);
      if (!!indicator.current_value) {
        valueInput.setAttribute("value", indicator.current_value);
      } else {
        valueInput.setAttribute("value", 0);
      }

      // If there was an error in the form data, the indicator value given by the user
      // is saved in currentActivityIndicator["indicator_value"]
      // we use this value as current value for the input.
      if (typeof(currentActivityIndicator) != "undefined") {
        if (!!currentActivityIndicator["indicator_value"]) {
          valueInput.selectedIndex = parseInt(currentActivityIndicator["indicator_value"]);
          valueInput.setAttribute("value", currentActivityIndicator["indicator_value"]);
        }
      }
    } else {
      // exsiting acivity indicator, hense its id is provided. No need to provide indicator_id
      idInput.setAttribute("id", "activity_activity_indicators_"+n+"_id");
      idInput.setAttribute("name", "activity[activity_indicators]["+n+"][id]");
      idInput.setAttribute("value", currentActivityIndicator["id"]);

      valueInput.selectedIndex = parseInt(currentActivityIndicator["indicator_value"]);
      valueInput.setAttribute("value", currentActivityIndicator["indicator_value"]);
    }

    inputFormElement.appendChild(idInput);

    let unitsText = indicator["units"];
    if (unitsText === null) {
      unitsText = "";
    };

    if (kind["type"] === "percentage") {
      unitsText = "% "+unitsText;
    }

    if (unitsText != "") {
      let inputGroup = document.createElement("div");
      inputGroup.classList.add("input-group");


      let inputUnits = document.createElement("div");
      inputUnits.classList.add("input-group-append");
      inputUnits.innerHTML = "<button class=\"btn btn-input-append\" type=\"button\">"+unitsText+"</button>";

      inputGroup.appendChild(valueInput);
      inputGroup.appendChild(inputUnits);

      inputFormElement.appendChild(inputGroup);
    } else {
      if (kind["type"] === "numeric") {
        inputFormElement.classList.add("col-md-6");
      }
      inputFormElement.appendChild(valueInput);
    }
    // inputFormElement.appendChild(inputLabel);
    // inputFormElement.appendChild(valueInput);
    // inputFormElement.appendChild(inputUnits);

    inputCol.appendChild(inputFormElement)

    return inputCol;
  },

  // OBSOLETE
  // _createIconCol(kind) {
  //   let iconCol = document.createElement("div");

  //   iconCol.classList.add("col-auto");
  //   iconCol.classList.add("ef-indicator-icon-col");
  //   iconCol.innerHTML="<i class='fas fa-"+kind["icon"]+" fa-2x text-gray-300 ef-indicator-icon ef-"+kind["type"] +"-icon' />"

  //   return iconCol;
  // },

  // <div class="card-header ef-indicator-header align-items-center d-flex mb-3">
  //   <div class="ef-i-circle ef-i-<%= @indicator.indicator_kind.type %>-circle align-items-center d-flex justify-content-center mr-2">
  //     <%= OgpEuskadiWeb.IndicatorView.indicator_icon(@indicator.indicator_kind.type) %>
  //   </div>
  //   <%= @indicator.title %>
  // </div>
  _createCardHeader(kind, indicator) {
    let headerElement = document.createElement("div");
    let iconElement = document.createElement("div");
    let titleElement = document.createElement("span");

    titleElement.innerHTML = indicator["title"];

    iconElement.classList.add("ef-i-circle", "ef-i-"+kind["type"]+"-circle", "align-items-center", "d-flex", "justify-content-center", "mr-2");
    iconElement.innerHTML="<i class='fas fa-"+kind["icon"]+" fa-2x text-gray-300 ef-indicator-icon ef-"+kind["type"] +"-icon' />";

    headerElement.classList.add("card-header", "ef-indicator-header", "align-items-center", "d-flex", "mb-3");
    headerElement.appendChild(iconElement);
    headerElement.appendChild(titleElement);

    return headerElement;
  }
}

export default ActivityForm;
