defmodule OgpEuskadi.Accounts do

  import Ecto.Query, warn: false
  alias OgpEuskadi.Repo

  alias OgpEuskadi.Accounts.User
  alias OgpEuskadi.Accounts.Credential

  def get_user!(id, prefix) do
    User
    |> Repo.get!(id, prefix: prefix)
    |> Repo.preload(:credential)
  end

  def get_user(id, prefix) do
    User
    |> Repo.get(id, prefix: prefix)
    |> Repo.preload(:credential)
  end

  def list_users(prefix) do
    User
    |> Repo.all(prefix: prefix)
    |> Repo.preload(:credential)
  end

  def change_user(%User{} = user) do
    User.changeset(user, %{})
  end

  def create_user(%{} = attrs, prefix) do
    %User{}
    |> User.changeset(attrs)
    |> Ecto.Changeset.cast_assoc(:credential, with: &Credential.changeset/2)
    |> Repo.insert(prefix: prefix)
  end

  def update_user(%User{} = user, attrs) do
    res = user |> User.changeset(attrs)

    # Update credentials only when a new password is provided.
    res = if (attrs["credential"]["password"] != "") do
      res |> Ecto.Changeset.cast_assoc(:credential, with: &Credential.changeset/2)
    else
      res
    end

    res |> Repo.update()
  end

  def delete_user(%User{} = user) do
    Repo.delete(user)
  end

  alias OgpEuskadi.Accounts.Credential

  @doc """
  Returns the list of credentials.

  ## Examples

      iex> list_credentials()
      [%Credential{}, ...]

  """
  def list_credentials(prefix) do
    Repo.all(Credential, prefix: prefix)
  end

  @doc """
  Gets a single credential.

  Raises `Ecto.NoResultsError` if the Credential does not exist.

  ## Examples

      iex> get_credential!(123)
      %Credential{}

      iex> get_credential!(456)
      ** (Ecto.NoResultsError)

  """
  def get_credential!(id, prefix), do: Repo.get!(Credential, id, prefix: prefix)

  @doc """
  Creates a credential.

  ## Examples

      iex> create_credential(%{field: value})
      {:ok, %Credential{}}

      iex> create_credential(%{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def create_credential(%{} = attrs, prefix) do
    %Credential{}
    |> Credential.changeset(attrs)
    |> Repo.insert(prefix: prefix)
  end

  @doc """
  Updates a credential.

  ## Examples

      iex> update_credential(credential, %{field: new_value})
      {:ok, %Credential{}}

      iex> update_credential(credential, %{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def update_credential(%Credential{} = credential, attrs) do
    credential
    |> Credential.changeset(attrs)
    |> Repo.update()
  end

  @doc """
  Deletes a Credential.

  ## Examples

      iex> delete_credential(credential)
      {:ok, %Credential{}}

      iex> delete_credential(credential)
      {:error, %Ecto.Changeset{}}

  """
  def delete_credential(%Credential{} = credential) do
    Repo.delete(credential)
  end

  @doc """
  Returns an `%Ecto.Changeset{}` for tracking credential changes.

  ## Examples

      iex> change_credential(credential)
      %Ecto.Changeset{source: %Credential{}}

  """
  def change_credential(%Credential{} = credential) do
    Credential.changeset(credential, %{})
  end

  def get_user_by_email(email, prefix) do
    from(u in User, join: c in assoc(u, :credential), where: c.email == ^email)
    |> Repo.one(prefix: prefix)
    |> Repo.preload(:credential)
  end

  def authenticate_by_email_and_pass(email, given_pass, prefix) do
    user = get_user_by_email(email, prefix)
    cond do
      user && Pbkdf2.verify_pass(given_pass, user.credential.password_hash) ->
        {:ok, user}
      user ->
        {:error, :unauthorized}
      true ->
        Pbkdf2.no_user_verify()
        {:error, :not_found}
    end
  end
end
