defmodule OgpEuskadi.Admin do
  @moduledoc """
    Admin context. Functions to manage tenants.
  """

  import Ecto.Query, warn: false

  alias OgpEuskadi.Repo

  alias OgpEuskadi.Site

  alias OgpEuskadi.Config
  alias OgpEuskadi.EntityLogo
  alias OgpEuskadi.Accounts

  def list_entities() do
    Site.Entity |> Repo.all
  end

  def get_entity!(id) do
    Site.Entity |> Repo.get!(id)
  end

  def create_entity(%{name: name, tenant: tenant_name, admin_email: admin_email} = attrs) do
    {:ok, tenant} = create_entity_tenant_and_users(%{name: name, tenant: tenant_name, admin_email: admin_email} = attrs)

    create_site_entity(tenant)
  end

  # Creates the entity tenant and users
  # See https://hexdocs.pm/triplex/Triplex.html#create/2
  def create_entity_tenant_and_users(%{name: name, tenant: tenant_name, admin_email: admin_email} = attrs) do
    Triplex.create_schema(tenant_name, Repo, fn(tenant, repo) ->
      Repo.transaction(fn ->
        {:ok, _} = Triplex.migrate(tenant, repo)
        prefix = Triplex.to_prefix(tenant)
        Accounts.create_user(%{username: "Admin", credential: %{email: admin_email, password: "ogp_obaba"}}, prefix)

        # the [`create_schema/3`](#create_schema/3) function must return `{:ok, "tenant"}`
        # if succeeded, and `Repo.transaction` transforms the function results
        # on this tuple
        tenant
      end)
    end)
  end

  def create_site_entity(tenant) do
    entity_config = Config.get_entity_config(Triplex.to_prefix(tenant))

    %Site.Entity{}
    |> Site.Entity.changeset(%{
      tenant: tenant,
      name_es: (entity_config.name_es || tenant),
      name_eu: (entity_config.name_eu || tenant),
      logo_path: EntityLogo.url({entity_config.logo, entity_config}, :thumb)})
    |> Repo.insert()
  end

  def update_site_entity(%Site.Entity{} = site_entity) do
    entity_config = Config.get_entity_config(Triplex.to_prefix(site_entity.tenant))

    site_entity
    |> Site.Entity.changeset(%{
      name_es: entity_config.name_es,
      name_eu: entity_config.name_eu,
      logo_path: EntityLogo.url({entity_config.logo, entity_config}, :thumb)})
    |> Repo.update()
  end

  def get_site_entity(tenant) do
    Site.Entity
    |> Repo.get_by(tenant: tenant)
  end

  def publish_site_entity(%Site.Entity{} = site_entity) do
    {:ok, now} = DateTime.now("Europe/Madrid")

    site_entity
    |> Site.Entity.changeset(%{published: true, published_at: now})
    |> Repo.update()
  end

  def unpublish_site_entity(%Site.Entity{} = site_entity) do
    site_entity
    |> Site.Entity.changeset(%{published: false, published_at: nil})
    |> Repo.update()
  end

end
