defmodule OgpEuskadi.Archive do
  @moduledoc """
  The Archive context.
  """

  import Ecto.Query, warn: false

  alias OgpEuskadi.Repo

  alias OgpEuskadi.Accounts
  alias OgpEuskadi.Archive.Photo

  @doc """
  Returns the list of photos.

  ## Examples

      iex> list_photos()
      [%Photo{}, ...]

  """
  def list_photos(prefix) do
    Repo.all(Photo, prefix: prefix)
  end

  @doc """
  Gets a single photo.

  Raises `Ecto.NoResultsError` if the Photo does not exist.

  ## Examples

      iex> get_photo!(123)
      %Photo{}

      iex> get_photo!(456)
      ** (Ecto.NoResultsError)

  """
  def get_photo!(id, prefix) do
    Photo
    |> Repo.get!(id, prefix: prefix)
    |> Repo.preload(:activity)
  end

  @doc """
  Creates a photo.

  ## Examples

      iex> create_photo(%Accounts.User{} = user, %{field: value})
      {:ok, %Photo{}}

      iex> create_photo(%{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def create_photo(%Accounts.User{} = user, %{} = attrs, prefix) do
    %Photo{}
    |> Ecto.put_meta(prefix: prefix) # necessary for the image storage dir
    |> Photo.changeset(attrs)
    |> Ecto.Changeset.put_assoc(:user, user)
    |> Repo.insert(prefix: prefix)
  end

  @doc """
  Updates a photo.

  ## Examples

      iex> update_photo(user, photo, %{field: new_value})
      {:ok, %Photo{}}

      iex> update_photo(user, photo, %{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def update_photo(user, %Photo{} = photo, attrs) do
    photo
    |> Repo.preload(:user)
    |> Photo.changeset(attrs)
    |> Ecto.Changeset.put_assoc(:user, user)
    |> Repo.update()
  end

  @doc """
  Deletes a Photo.

  ## Examples

      iex> delete_photo(photo)
      {:ok, %Photo{}}

      iex> delete_photo(photo)
      {:error, %Ecto.Changeset{}}

  """
  def delete_photo(%Photo{} = photo) do
    Repo.delete(photo)
  end

  @doc """
  Returns an `%Ecto.Changeset{}` for tracking photo changes.

  ## Examples

      iex> change_photo(photo)
      %Ecto.Changeset{source: %Photo{}}

  """
  def change_photo(%Photo{} = photo) do
    Photo.changeset(photo, %{})
  end

  alias OgpEuskadi.Archive.Document

  @doc """
  Returns the list of documents.

  ## Examples

      iex> list_documents()
      [%Document{}, ...]

  """
  def list_documents(prefix) do
    Repo.all(Document, prefix: prefix)
  end

  @doc """
  Gets a single document.

  Raises `Ecto.NoResultsError` if the Document does not exist.

  ## Examples

      iex> get_document!(123)
      %Document{}

      iex> get_document!(456)
      ** (Ecto.NoResultsError)

  """
  def get_document!(id, prefix) do
    Document
    |> Repo.get!(id, prefix: prefix)
    |> Repo.preload(:activity)
  end

  @doc """
  Creates a document.

  ## Examples

      iex> create_document(%Accounts.User{} = user, %{field: value})
      {:ok, %Document{}}

      iex> create_document(%{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def create_document(%Accounts.User{} = user, %{} = attrs, prefix) do
    %Document{}
    |> Ecto.put_meta(prefix: prefix) # necessary for the file storage dir
    |> Document.changeset(attrs)
    |> Ecto.Changeset.put_assoc(:user, user)
    |> Repo.insert(prefix: prefix)
  end

  @doc """
  Updates a document.

  ## Examples

      iex> update_document(%Accounts.User{} = user, document, %{field: new_value})
      {:ok, %Document{}}

      iex> update_document(document, %{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def update_document(user, %Document{} = document, attrs) do
    document
    |> Repo.preload(:user)
    |> Document.changeset(attrs)
    |> Ecto.Changeset.put_assoc(:user, user)
    |> Repo.update()
  end

  @doc """
  Deletes a Document.

  ## Examples

      iex> delete_document(document)
      {:ok, %Document{}}

      iex> delete_document(document)
      {:error, %Ecto.Changeset{}}

  """
  def delete_document(%Document{} = document) do
    Repo.delete(document)
  end

  @doc """
  Returns an `%Ecto.Changeset{}` for tracking document changes.

  ## Examples

      iex> change_document(document)
      %Ecto.Changeset{source: %Document{}}

  """
  def change_document(%Document{} = document) do
    Document.changeset(document, %{})
  end
end
