defmodule OgpEuskadi.Archive.Document do
  use Ecto.Schema
  use Arc.Ecto.Schema
  use OgpEuskadi.Translator, translates: [:caption, :url]

  import Ecto.Changeset

  alias OgpEuskadi.Accounts.User
  alias OgpEuskadi.Monitoring.Activity

  schema "documents" do
    field :file, OgpEuskadi.Document.Type
    field :url_es, :string
    field :url_eu, :string
    field :caption_es, :string
    field :caption_eu, :string

    field :caption, :string, virtual: true
    field :url, :string, virtual: true

    field :remove_file, :boolean, virtual: true
    field :file_cache, :string, virtual: true

    belongs_to :activity, Activity
    belongs_to :user, User

    timestamps()
  end

  @doc false
  def changeset(document, attrs) do

    attrs = attrs |> set_file_from_cache("file_cache", "file")

    document
    |> cast(attrs, [:caption_es, :caption_eu, :url_es, :url_eu, :activity_id, :remove_file, :file_cache])
    |> cast_attachments(attrs, [:file], [allow_paths: true])
    |> validate_required([:activity_id, :caption_es])
    |> may_be_remove_file
    |> cache_uploaded_file(attrs["file"], :file_cache)
    |> validate_file_or_url_provided
  end

  # Reference https://code.tutsplus.com/tutorials/working-with-file-system-in-elixir--cms-28869
  def remove_cached_files(seconds) do
    IO.puts "remove old cached files from #{cache_dir}"

    now = DateTime.utc_now()
    case File.ls(cache_dir) do
      {:ok, dirs} ->
        dirs
        |> Enum.map(&(Path.join(cache_dir, &1)))
        |> Stream.filter(fn full_path ->
          atime_utc = DateTime.from_naive!(NaiveDateTime.from_erl!(File.lstat!(full_path).atime), "Etc/UTC")
          DateTime.diff(now, atime_utc) > seconds
        end)
        |> Enum.map(fn dir_path ->
          File.rm_rf!(dir_path)
          dir_path
        end)

      _ ->
        []
    end
  end

  defp cache_dir do
    "/tmp/ogpe"
  end

  defp set_file_from_cache(attrs, file_cache_field, file_field) do
    case attrs[file_cache_field] do
      nil ->
        attrs

      cached_file_path ->
        attrs
        |> Map.put_new(file_field, Path.join(cache_dir, cached_file_path))
    end
  end

  # Cache uplodaded file in a temporary dir.
  defp cache_uploaded_file(%Ecto.Changeset{} = changeset, %Plug.Upload{} = uploaded_file, field_name) do

    # first remove cached files older than 10 minutes
    remove_cached_files(600)

    # cache the current file
    cached_file_dir = :crypto.strong_rand_bytes(20) |> Base.encode32()
    cached_file_name = uploaded_file.filename

    cached_file_path =
      case File.mkdir_p(Path.join(cache_dir, cached_file_dir)) do
        :ok ->
          Path.join(cached_file_dir, cached_file_name)

        {:error, _} ->
          cached_file_name
      end

    case File.rename(uploaded_file.path, Path.join(cache_dir, cached_file_path)) do
      :ok ->
        changeset
        |> put_change(field_name, cached_file_path)

      _ ->
        changeset
    end
  end

  defp cache_uploaded_file(%Ecto.Changeset{} = changeset, _uploaded_file, _field_name) do
    changeset
  end

  defp may_be_remove_file(%Ecto.Changeset{} = changeset) do
    if get_change(changeset, :remove_file) do
      changeset
      |> change(%{file: nil})
    else
      changeset
    end
  end

  # Reference: https://toranbillups.com/blog/archive/2019/02/25/writing-custom-validations-with-ecto-3/
  # We use get_field to take the changed value or the current data value (https://hexdocs.pm/ecto/Ecto.Changeset.html#get_field/3)
  defp validate_file_or_url_provided(%Ecto.Changeset{} = changeset) do
    file = get_field(changeset, :file)
    url = get_field(changeset, :url_es)

    cond do
      !!url && !!file ->
        if is_nil(get_change(changeset, :remove_file)) do
          changeset
          |> add_error(:file, "do not provide file when url is provided")
          |> add_error(:url_es, "do not provide url when file is provided")
        else
          changeset
          |> add_error(:remove_file, "must remove the file to add url")
        end

      !url && !file ->
        changeset
        |> add_error(:file, "file or url must be provided")
        |> add_error(:url_es, "file or url must be provided")

      true ->
        changeset
    end
  end
end
