defmodule OgpEuskadi.Basic do
  @moduledoc """
  The Basic context.
  """

  import Ecto.Query, warn: false
  alias OgpEuskadi.Repo

  alias OgpEuskadi.Basic.Topic
  alias OgpEuskadi.Basic.IndicatorKind
  alias OgpEuskadi.Basic.Ods

  @doc """
  Returns the list of topics.

  ## Examples

      iex> list_topics()
      [%Topic{}, ...]

  """
  def list_topics(prefix) do
    Repo.all(Topic, prefix: prefix)
  end

  @doc """
  Gets a single topic.

  Raises `Ecto.NoResultsError` if the Topic does not exist.

  ## Examples

      iex> get_topic!(123, "prefix")
      %Topic{}

      iex> get_topic!(456, "prefix")
      ** (Ecto.NoResultsError)

  """
  def get_topic!(id, prefix), do: Repo.get!(Topic, id, prefix: prefix)

  @doc """
  Creates a topic.

  ## Examples

      iex> create_topic(%{field: value}, "prefix")
      {:ok, %Topic{}}

      iex> create_topic(%{field: bad_value}, "prefix")
      {:error, %Ecto.Changeset{}}

  """
  def create_topic(attrs, prefix) do
    %Topic{}
    |> Topic.changeset(attrs)
    |> Repo.insert(prefix: prefix)
  end

  @doc """
  Updates a topic.

  ## Examples

      iex> update_topic(topic, %{field: new_value})
      {:ok, %Topic{}}

      iex> update_topic(topic, %{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def update_topic(%Topic{} = topic, attrs) do
    topic
    |> Topic.changeset(attrs)
    |> Repo.update()
  end

  @doc """
  Deletes a Topic.

  ## Examples

      iex> delete_topic(topic)
      {:ok, %Topic{}}

      iex> delete_topic(topic)
      {:error, %Ecto.Changeset{}}

  """
  def delete_topic(%Topic{} = topic) do
    Repo.delete(topic, prefix: Ecto.get_meta(topic, :prefix))
  end

  @doc """
  Returns an `%Ecto.Changeset{}` for tracking topic changes.

  ## Examples

      iex> change_topic(topic)
      %Ecto.Changeset{source: %Topic{}}

  """
  def change_topic(%Topic{} = topic) do
    Topic.changeset(topic, %{})
  end

  # Indicator kinds

  @doc """
  Returns the list of indicator kinds.

  ## Examples

      iex> list_indicator_kinds()
      [%IndicatorKind{}, ...]

  """
  def list_indicator_kinds(prefix) do
    Repo.all(IndicatorKind, prefix: prefix)
  end

  @doc """
  Returns the list of indicator of type "phases" kinds.

  ## Examples

      iex> list_phases_indicator_kinds()
      [%IndicatorKind{}, ...]

  """
  def list_phases_indicator_kinds(prefix) do
     q = from ik in IndicatorKind,
       where: ik.type == "phases",
       order_by: ik.position

    Repo.all(q, prefix: prefix)
  end

  def get_indicator_phases_options(prefix) do
    list_phases_indicator_kinds(prefix)
    |> Enum.map(fn ik -> [key: ik.name_es, value: ik.id] end)
  end


  @doc """
  Gets a single indicator kind.

  Raises `Ecto.NoResultsError` if the IndicatorKind does not exist.

  ## Examples

      iex> get_indicator_kind!(prefix, 123)
      %IndicatorKind{}

      iex> get_indicator_kind(prefix, 456)
      ** (Ecto.NoResultsError)

  """
  def get_indicator_kind!(id, prefix), do: Repo.get!(IndicatorKind, id, prefix: prefix)


  @doc """
  Creates indicator kind.

  ## Examples

      iex> create_indicator_kind(prefix, %{field: value})
      {:ok, %IndcatorKind{}}

      iex> create_indicator_kind(prefix, %{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def create_indicator_kind(attrs, prefix) do
    %IndicatorKind{}
    |> IndicatorKind.changeset(attrs)
    |> Repo.insert(prefix: prefix)
  end

  @doc """
  Updates indicator kind.

  ## Examples

      iex> update_indicator_kind(indicator_kind, %{field: new_value})
      {:ok, %IndicatorKind{}}

      iex> update_indicator_kind(indicator_kind, %{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def update_indicator_kind(%IndicatorKind{} = indicator_kind, attrs) do
    indicator_kind
    |> IndicatorKind.changeset(attrs)
    |> Repo.update()
  end


  # ODS

  @doc """
  Returns the list of ods.

  ## Examples

      iex> list_ods()
      [%Ods{}, ...]

  """
  def list_ods(prefix) do
    Repo.all(Ods, prefix: prefix)
  end

end
