defmodule OgpEuskadi.Config do
  @moduledoc """
  The Config context.
  """

  import Ecto.Query, warn: false
  alias OgpEuskadi.Repo

  alias OgpEuskadi.Config.EntityConfig
  alias OgpEuskadi.Config.Subdivision
  alias OgpEuskadi.Config.EntityService

  @doc """
  Gets a single entity_config.

  Raises `Ecto.NoResultsError` if the Entity config does not exist.

  ## Examples

      iex> get_entity_config("ogpe_obaba", 123)
      %EntityConfig{}

      iex> get_entity_config("ogpe_obaba", 456)
      ** (Ecto.NoResultsError)

  """
  def get_entity_config(prefix) do
    case Repo.all(EntityConfig, prefix: prefix) do
      [] -> %EntityConfig{}
      [entity_config] -> entity_config
    end
  end

  @doc """
  Creates a entity_config.

  ## Examples

      iex> create_entity_config("ogpe_obaba", %{field: value})
      {:ok, %EntityConfig{}}

      iex> create_entity_config("ogpe_obaba", %{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def create_entity_config(%{} = attrs, prefix) do
    %EntityConfig{}
    |> Ecto.put_meta(prefix: prefix) # necessary for the logo storage dir
    |> EntityConfig.changeset(attrs)
    |> Repo.insert(prefix: prefix)
  end

  @doc """
  Updates a entity_config.

  ## Examples

      iex> update_entity_config("ogpe_obaba", entity_config, %{field: new_value})
      {:ok, %EntityConfig{}}

      iex> update_entity_config("ogpe_obaba", entity_config, %{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def update_entity_config(%EntityConfig{} = entity_config, attrs, prefix) do
    if !!entity_config.id do
        entity_config
        |> EntityConfig.changeset(attrs)
        |> Repo.update()
    else
      create_entity_config(attrs, prefix)
    end
  end

  @doc """
  Returns an `%Ecto.Changeset{}` for tracking entity_config changes.

  ## Examples

      iex> change_entity_config(entity_config)
      %Ecto.Changeset{source: %EntityConfig{}}

  """
  def change_entity_config(%EntityConfig{} = entity_config) do
    EntityConfig.changeset(entity_config, %{})
  end


  @doc """
  Returns the list of subdivisions.

  ## Examples

      iex> list_subdivisions()
      [%Subdivision{}, ...]

  """
  def list_subdivisions(prefix) do
    from(s in Subdivision, order_by: s.name_es)
    |> Repo.all(prefix: prefix)
  end

  @doc """
  Gets a single subdivision.

  Raises `Ecto.NoResultsError` if the Subdivision does not exist.

  ## Examples

      iex> get_subdivision!(123)
      %Subdivision{}

      iex> get_subdivision!(456)
      ** (Ecto.NoResultsError)

  """
  def get_subdivision!(id, prefix), do: Repo.get!(Subdivision, id, prefix: prefix)


  @doc """
  Create a subdivision.
  ## Examples
        iex> create_subdivision(%{field: value}, prefix)
        {:ok, %Subdivision{}}

        iex> create_subdivision(%{field: bad_value}, prefix)
        {:error, %Ecto.Changeset{}}
  """
  def create_subdivision(attrs, prefix) do
    %Subdivision{}
    |> Subdivision.changeset(attrs)
    |> Repo.insert(prefix: prefix)
  end

  @doc """
  Updates a subdivision.

  ## Examples

      iex> update_subdivision(subdivision, %{field: new_value})
      {:ok, %Subdivision{}}

      iex> update_subdivision(subdivision, %{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def update_subdivision(%Subdivision{} = subdivision, attrs) do
    subdivision
    |> Subdivision.changeset(attrs)
    |> Repo.update()
  end

  @doc """
  Deletes a subdivision.

  ## Examples

      iex> delete_subdivision(subdivision)
      {:ok, %Goal{}}

      iex> delete_subdivision(subdivision)
      {:error, %Ecto.Changeset{}}

  """
  def delete_subdivision(%Subdivision{} = subdivision) do
    Repo.delete(subdivision, prefix: Ecto.get_meta(subdivision, :prefix))
  end


  @doc """
  Returns an `%Ecto.Changeset{}` for tracking subdivision changes.
  ## Examples

      iex> change_subdivision(goal)
      %Ecto.Changeset{source: %Subdivision{}}

  """
  def change_subdivision(%Subdivision{} = subdivision) do
    Subdivision.changeset(subdivision, %{})
  end


  @doc """
  Returns the list of entity services.

  ## Examples

      iex> list_entity_services()
      [%EntityService{}, ...]

  """
  def list_entity_services(prefix) do
    from(s in EntityService, order_by: s.position)
    |> Repo.all(prefix: prefix)
  end

  @doc """
  Gets a single entity service.

  Raises `Ecto.NoResultsError` if the EntityService does not exist.

  ## Examples

      iex> get_entity_service!(123)
      %EntityService{}

      iex> get_entity_service!(456)
      ** (Ecto.NoResultsError)

  """
  def get_entity_service!(id, prefix), do: Repo.get!(EntityService, id, prefix: prefix)


  @doc """
  Create an entity service.
  ## Examples
        iex> create_entity_service(%{field: value}, prefix)
        {:ok, %EntityService{}}

        iex> create_entity_service(%{field: bad_value}, prefix)
        {:error, %Ecto.Changeset{}}
  """
  def create_entity_service(attrs, prefix) do
    %EntityService{}
    |> EntityService.changeset(attrs)
    |> Repo.insert(prefix: prefix)
  end

  @doc """
  Updates an entity service.

  ## Examples

      iex> update_entity_service(entity_service, %{field: new_value})
      {:ok, %EntityService{}}

      iex> update_entity_service(entity_service, %{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def update_entity_service(%EntityService{} = entity_service, attrs) do
    entity_service
    |> EntityService.changeset(attrs)
    |> Repo.update()
  end

  @doc """
  Deletes an entity service.

  ## Examples

      iex> delete_entity_service(entity_service)
      {:ok, %EntityService{}}

      iex> delete_entity_service(subdivision)
      {:error, %Ecto.Changeset{}}

  """
  def delete_entity_service(%EntityService{} = entity_service) do
    Repo.delete(entity_service, prefix: Ecto.get_meta(entity_service, :prefix))
  end


  @doc """
  Returns an `%Ecto.Changeset{}` for tracking entity service changes.
  ## Examples

      iex> change_entity_service(entity_service)
      %Ecto.Changeset{source: %EntityService{}}

  """
  def change_entity_service(%EntityService{} = entity_service) do
    EntityService.changeset(entity_service, %{})
  end


end
