defmodule OgpEuskadi.Monitoring do
  @moduledoc """
  The Monitoring context.
  """

  import Ecto.Query, warn: false
  alias OgpEuskadi.Repo

  alias OgpEuskadi.Monitoring.ActMonitoring
  alias OgpEuskadi.Monitoring.Activity
  alias OgpEuskadi.Monitoring.Indicator
  alias OgpEuskadi.Monitoring.ActivityIndicator


  @doc """
  Returns the list of indicators for given act_id
  """
  def list_act_indicators(act_id, prefix) do
    q = from ind in Indicator,
      where: [act_id: ^act_id],
      preload: [:indicator_kind, act: :goal]

    Repo.all(q, prefix: prefix)
  end

  @doc """
  Returns the list of activities.

  ## Examples

      iex> list_activities()
      [%Activity{}, ...]

  """
  def list_activities(prefix) do
    # from(a in Activity, order_by: [desc: a.date, desc: a.updated_at])
    # |> Repo.all(prefix: prefix)
    # |> Repo.preload([
    #   activity_indicators: activity_indicators_query,
    #   act: :goal])
    list_activities_query
    |> Repo.all(prefix: prefix)
  end

  @doc """
  Returns the iquery for the list of activities. Suitable for paginating with Paginator

  ## Examples

      iex> list_activities_query()
      [%Activity{}, ...]

  """
  def list_activities_query do
    ai_query = activity_indicators_query
    from a in Activity,
      order_by: [desc: a.date, desc: a.updated_at],
      preload: [ activity_indicators: ^ai_query, act: :goal]
  end

  @doc """
  Returns the list of activities.

  ## Examples

      iex> list_activities()
      [%Activity{}, ...]

  """
  def list_activities_with_archive(prefix) do
    from(a in Activity, order_by: [desc: a.date, desc: a.updated_at])
    |> Repo.all(prefix: prefix)
    |> Repo.preload([
      :photos,
      :documents,
      activity_indicators: activity_indicators_query,
      act: :goal])
  end

  @doc """
  Gets a single activity.

  Raises `Ecto.NoResultsError` if the Activity does not exist.

  ## Examples

      iex> get_activity!(123)
      %Activity{}

      iex> get_activity!(456)
      ** (Ecto.NoResultsError)

  """
  def get_activity!(id, prefix) do
    Repo.get!(Activity, id, prefix: prefix)
    |> Repo.preload([activity_indicators: activity_indicators_query, act: :goal])
  end

  @doc """
  Gets a single activity with photos and documents.

  Raises `Ecto.NoResultsError` if the Activity does not exist.

  ## Examples

      iex> get_activity_with_archive!(123)
      %Activity{}

      iex> get_activity_with_archive!(456)
      ** (Ecto.NoResultsError)

  """
  def get_activity_with_archive!(id, prefix) do
    Repo.get!(Activity, id, prefix: prefix)
    |> Repo.preload([
      act: :goal,
      photos: [],
      documents: [],
      activity_indicators: activity_indicators_query
    ])
  end


  def activities_with_associations_query(activities_queryable) do

    from(a in activities_queryable,
         order_by: [desc: a.date, desc: a.updated_at],
         preload: [
            photos: [],
            documents: [],
            act: [],
            activity_indicators: ^activity_indicators_query
         ]
    )
  end

  def activity_indicators_query do
    from(ai in ActivityIndicator, order_by: ai.indicator_id, preload: [indicator: :indicator_kind])
  end

  def preload_activity_act(activity) do
    activity |> Repo.preload(:act)
  end

  # Create/update activity

  @doc """
  Returns the most recent activity for act_id and prefix
  """
  def get_last_published_act_activity(act) do
    query =
      from a in Activity,
        where: a.act_id==^act.id and a.draft == ^false,
        order_by: [desc: a.date, desc: a.updated_at]

    Repo.one(query |> first(), prefix: Ecto.get_meta(act, :prefix))
  end

  @doc """
  Returns the date of the last act activity or nil if act has no activities.
  """
  def get_last_published_act_activity_date(act) do
    case get_last_published_act_activity(act) do

      %Activity{} = last_activity ->
        last_activity.date

      nil ->
        nil
    end
  end

  @doc """
    Returns indicator value taken from the last published activity.
   """
  def get_indicator_current_value_from_activities(indicator) do
    activity_indicators_query =
      from ai in ActivityIndicator,
        join: a in assoc(ai, :activity),
        where: ai.indicator_id==^indicator.id and a.draft==^false,
        order_by: [desc: a.date, desc: a.updated_at]

    case Repo.one(activity_indicators_query  |> first(), prefix: Ecto.get_meta(indicator, :prefix)) do
      %ActivityIndicator{} = activity_indicator -> activity_indicator.indicator_value
      _ -> nil
    end
  end

  def get_act_monitoring!(act_id, prefix) do
    ActMonitoring |> Repo.get!(act_id, prefix: prefix)
  end

  def update_act_monitoring(act) do
    act
    |> ActMonitoring.changeset(%{"last_activity_date" => get_last_published_act_activity_date(act)})
    |> Repo.update
  end

  def update_act_last_activity_date(_repo, %{activity: activity}) do
    get_act_monitoring!(activity.act_id, Ecto.get_meta(activity, :prefix))
    |> update_act_monitoring()
  end

  def update_indicators_current_value(_repo, %{activity: activity}) do
    ai_updates =
      activity.activity_indicators
      |> Enum.map( fn ai ->
        ai = ai |> Repo.preload(:indicator)
        ai.indicator
        |> Ecto.Changeset.change(%{current_value: get_indicator_current_value_from_activities(ai.indicator)})
        |> Repo.update()
      end)

    {:ok, ai_updates}
  end

  @doc """
  Creates a activity.

  Sets the value for last_activity_date on the related act.

  Sets the values for the related act indicators.


  ## Examples

      iex> create_activity(%{field: value})
      {:ok, %Activity{}}

      iex> create_activity(%{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def create_activity(attrs \\ %{}, prefix) do
      Ecto.Multi.new()
      |> Ecto.Multi.insert(:activity, Activity.changeset(%Activity{}, attrs), prefix: prefix)
      |> Ecto.Multi.run(:act_monitoring, &update_act_last_activity_date/2)
      |> Ecto.Multi.run(:activity_indicators, &update_indicators_current_value/2)
      |> Repo.transaction()
  end

  @doc """
  Updates a activity.

  Sets the value for last_activity_date on the related act.

  Sets the values for the related act indicators.

  ## Examples

      iex> update_activity(activity, %{field: new_value})
      {:ok, %Activity{}}

      iex> update_activity(activity, %{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def update_activity(%Activity{} = activity, attrs) do
    Ecto.Multi.new()
    |> Ecto.Multi.update(:activity, Activity.changeset(activity, attrs))
    |> Ecto.Multi.run(:act_monitoring, &update_act_last_activity_date/2)
    |> Ecto.Multi.run(:activity_indicators, &update_indicators_current_value/2)
    |> Repo.transaction()
  end

  @doc """
  Deletes a Activity.

  ## Examples

      iex> delete_activity(activity)
      {:ok, %Activity{}}

      iex> delete_activity(activity)
      {:error, %Ecto.Changeset{}}

  """
  def delete_activity(%Activity{} = activity) do
    Ecto.Multi.new()
    |> Ecto.Multi.delete(:activity, activity)
    |> Ecto.Multi.run(:act_monitoring, &update_act_last_activity_date/2)
    |> Ecto.Multi.run(:activity_indicators, &update_indicators_current_value/2)
    |> Repo.transaction()
  end

  @doc """
  Returns an `%Ecto.Changeset{}` for tracking activity changes.

  ## Examples

  iex> change_activity(activity, %{})
      %Ecto.Changeset{source: %Activity{}}

  """
  def change_activity(%Activity{} = activity, attrs \\ %{}) do
    Activity.changeset(activity, attrs)
  end

  # Indicator

  @doc """
  Gets a single indicator.

  Raises `Ecto.NoResultsError` if the Indicator does not exist.

  ## Examples

  iex> get_indicator(123, prefix)
      %Indicator{}

      iex> get_indicator(456, prefix)
      ** (Ecto.NoResultsError)

  """
  def get_indicator!(id, prefix) do
    Repo.get!(Indicator, id, prefix: prefix)
    |> Repo.preload([:activity_indicators, :indicator_kind, act: :goal])
  end


  @doc """
  Returns an `%Ecto.Changeset{}` for tracking indicator changes.

  ## Examples

  iex> change_indicator(activity, %{})
      %Ecto.Changeset{source: %Indicator{}}

  """
  def change_indicator(%Indicator{} = indicator, attrs \\ %{}) do
    Indicator.changeset(indicator, attrs)
  end

  @doc """
  Creates an indicator and sets current value to 0.

  ## Examples

  iex> create_indicator(%{field: value}, prefix)
      {:ok, %Indicator{}}

      iex> create_indicator(%{field: bad_value}, prefix)
      {:error, %Ecto.Changeset{}}

  """
  def create_indicator(attrs, prefix) do
    %Indicator{}
    |> Indicator.changeset(attrs)
    |> Ecto.Changeset.change(%{current_value: 0})
    |> Repo.insert(prefix: prefix)
  end

  @doc """
  Updates an indicator. NOTE: current value is not updated.

  ## Examples

      iex> update_indicator(indicator, %{field: new_value})
      {:ok, %Indicator{}}

      iex> update_indicator(indicator, %{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def update_indicator(%Indicator{} = indicator, attrs) do
    indicator
    |> Indicator.changeset(attrs)
    |> Repo.update()
  end

  @doc """
  Deletes an Indicator.

  ## Examples

      iex> delete_indicator(indicator)
      {:ok, %Act{}}

      iex> delete_indicator(indicator)
      {:error, %Ecto.Changeset{}}

  """
  def delete_indicator(%Indicator{} = indicator) do
    indicator
    |> Ecto.Changeset.change
    |> Ecto.Changeset.no_assoc_constraint(:activity_indicators)
    |> Repo.delete()
  end

end
