defmodule OgpEuskadi.Overview do
  @moduledoc """
  The Overview context. Content for the public part of the application.
  The locale is passed as parameter as the content is show for a specific locale.
  """

  import Ecto.Query, warn: false
  import OgpEuskadi.QueryableTools

  alias OgpEuskadi.Repo

  alias OgpEuskadi.Translator

  alias OgpEuskadi.Basic.Topic
  alias OgpEuskadi.Basic.Ods
  alias OgpEuskadi.Config.EntityService
  alias OgpEuskadi.Config.Subdivision
  alias OgpEuskadi.Plan
  alias OgpEuskadi.Plan.Goal
  alias OgpEuskadi.Plan.Act
  alias OgpEuskadi.Plan.ActTopic
  alias OgpEuskadi.Plan.ActOds
  alias OgpEuskadi.Plan.ActSubdivision
  alias OgpEuskadi.Monitoring
  alias OgpEuskadi.Monitoring.Activity

  defp published_goals_query() do
    from(g in Goal, order_by: g.position) |> filter_published()
  end

  defp published_acts_query() do
    from(a in Act) |> filter_published()
  end

  defp published_activities_query() do
    from(activity in Activity, join: act in Act, on: activity.act_id == act.id and act.draft==^false)
    |> filter_published()
  end

  defp get_items_by_ids(queryable, ids) do
    from q in queryable, where: q.id in ^ids
  end

  defp list_publised_goals(prefix) do
    published_goals_query() |> Repo.all(prefix: prefix)
  end

  defp get_published_goal!(id, prefix) do
    published_goals_query() |> Repo.get!(id, prefix: prefix)
  end

  defp get_published_act!(id, prefix) do
    published_acts_query() |> Repo.get!(id, prefix: prefix)
  end

  # ORIGINAL
  ## def activity_counters_query(queryable, prefix) do
  ##   from act in queryable,
  ##     left_join: activity in subquery(published_actvities_query(), prefix: prefix), on: (act.id == activity.act_id),
  ##     group_by: [act.id, activity.act_id],
  ##     select: {act, count(activity.act_id)},
  ##     order_by: act.id,
  ##     preload: [:goal, :topics, indicators: :indicator_kind]
  ## end
  # /ORIGINAL


  def activity_counters_query(queryable, prefix) do
    from act in queryable,
      left_join: activity in subquery(published_actvities_query(), prefix: prefix), on: (act.id == activity.act_id),
      group_by: [act.id, activity.act_id],
      select: {act, count(activity.act_id)},
      preload: [:goal, :topics, :subdivisions, :ods, indicators: :indicator_kind]
  end

  #
  #  Goals
  #

  def list_goals_with_actions_and_activities(locale, prefix) do
    list_publised_goals(prefix)
    |> Repo.preload([acts: {published_acts_query() |> order_by_position(), [:activities]}])
    |> Translator.translate(locale)
  end

  def get_goal!(id, locale, prefix) do
    get_published_goal!(id, prefix)
    # |> Repo.preload([ods: (from o in Ods) |> order_by_position()])
    |> Translator.translate(locale)
  end

  def get_goal_acts_with_activity_counters(goal, locale) do
    Plan.goal_acts_query(goal)
    |> filter_published()
    |> Plan.acts_with_activity_counters_query
    |> Repo.all(prefix: Ecto.get_meta(goal, :prefix))
    |> Translator.translate(locale)
  end

  #
  # Topics
  #
  def get_topic!(id, locale, prefix) do
    Topic
    |> Repo.get!(id, prefix: prefix)
    |> Translator.translate(locale)
  end

  def list_topics_with_acts_counter(prefix) do
    q = from t in Topic,
        left_join: a in assoc(t, :act_topics),
        group_by: [t.id, a.topic_id],
        select: {t, count(a.topic_id)},
        order_by: t.id

    Repo.all(q, prefix: prefix)
  end

  def published_actvities_query() do
    from(a in Activity, where: a.draft==^false)
  end

  def topic_act_activites_counter_query(prefix) do
    from at in ActTopic,
      join: act in Act, on: (at.act_id == act.id and act.draft==^false),
      left_join: activity in Activity, on: (act.id == activity.act_id and activity.draft==^false),
      group_by: [at.topic_id, at.act_id],
      select: %{topic_id: at.topic_id, act_id: at.act_id, activities_count: count(activity.id)}
  end

  def list_topics_with_acts_and_activities_counter(locale, prefix) do
    title_for_sort = String.to_atom("title_#{locale}")
    q = from t in Topic,
      join: topic_act_activites_counter in subquery(topic_act_activites_counter_query(prefix), prefix: prefix),
      on: topic_act_activites_counter.topic_id == t.id,
      group_by: [t.id, topic_act_activites_counter.topic_id],
      select: {t, count(topic_act_activites_counter.act_id), sum(topic_act_activites_counter.activities_count)},
      order_by: [^title_for_sort]

    Repo.all(q, prefix: prefix)
    |> Translator.translate(locale)
  end

  def join_act_topic(queryable, topic) do
    from act in queryable,
      inner_join: at in ActTopic, on: ((act.id == at.act_id) and (at.topic_id == ^topic.id))
  end

  def join_topic_activities(queriable, topic_id) do
    from activity in queriable,
      inner_join: at in ActTopic, on: (activity.act_id == at.act_id) and (at.topic_id == ^topic_id)
  end

  def list_topic_acts_with_activity_counters(topic, locale) do
    prefix = Ecto.get_meta(topic, :prefix)

    published_acts_query()
    |> join_act_topic(topic)
    |> activity_counters_query(prefix)
    |> order_acts_by_last_activity_date()
    |> Repo.all(prefix: prefix)
    |> Translator.translate(locale)
  end

  @doc """
  Topics activities list order by thier date, the newest first.
  """
  def list_topic_activities(topic, locale) do
    # NOTE: activities preload with custom query does not respect order_by. It adds additional order by act.id
    # topic
    #   |> Repo.preload([activities: Monitoring.activities_with_associations_query(published_activities_query())])

    published_activities_query()
    |> join_topic_activities(topic.id)
    |> Monitoring.activities_with_associations_query()
    |> Repo.all(prefix: Ecto.get_meta(topic, :prefix))
    |> Translator.translate(locale)
  end

  #
  # Subdivisions
  #
  def subdivision_act_activites_counter_query(prefix) do
    from as in ActSubdivision,
      join: act in Act, on: (as.act_id == act.id and act.draft==^false),
      left_join: activity in Activity, on: (act.id == activity.act_id and activity.draft==^false),
      group_by: [as.subdivision_id, as.act_id],
      select: %{subdivision_id: as.subdivision_id, act_id: as.act_id, activities_count: count(activity.id)}
  end

  def list_subdivisions_with_acts_and_activities_counter(locale, prefix) do
    name_for_sort = String.to_atom("name_#{locale}")
    q = from s in Subdivision,
      join: subdivision_act_activites_counter in subquery(subdivision_act_activites_counter_query(prefix), prefix: prefix),
      on: subdivision_act_activites_counter.subdivision_id == s.id,
      group_by: [s.id, subdivision_act_activites_counter.subdivision_id],
      select: {s, count(subdivision_act_activites_counter.act_id), sum(subdivision_act_activites_counter.activities_count)},
      order_by: [^name_for_sort, :name_es]

    Repo.all(q, prefix: prefix)
    |> Translator.translate(locale)
  end

  def get_all_subdivisions_counters(prefix) do
    activities_counters_list = from(act in OgpEuskadi.Plan.Act,
      left_join: activity in assoc(act, :activities),
      where: act.all_subdivisions == true and act.draft == ^false,
      group_by: act.id,
      select: {act.id, count(activity.id)})
    |> Repo.all(prefix: prefix)

    acts_counter = length(activities_counters_list)
    activities_counter = Enum.reduce(activities_counters_list, 0, fn {_act_id, activities_count}, acc -> acc + activities_count end)


    {acts_counter, activities_counter}
  end

  def get_subdivision!(id, locale, prefix) do
    Subdivision
    |> Repo.get!(id, prefix: prefix)
    |> Translator.translate(locale)
  end

  def all_subdivisions_acts_query(queryable) do
    from(item in queryable, where: item.all_subdivisions == ^true)
  end

  def join_act_subdivision(queryable, subdivision) do
    from act in queryable,
      inner_join: asubd in ActSubdivision, on: ((act.id == asubd.act_id) and (asubd.subdivision_id == ^subdivision.id))
  end

  @doc """
  Returns the list with the acts that has assigned the subdivision "subvidision" or have all_subdivisions set to true.

  The need to use order by substring requires two queries: first the list woth acts_ids is taken
  and then the data and activities counter for the selected ids are taken.

  When union_all is used, the order_by can not contain functions and therefore we can not use substring.
  """
  def list_subdivision_acts_with_activity_counters(subdivision, locale) do
    prefix = Ecto.get_meta(subdivision, :prefix)

    all_subdvisions_acts_with_activity_counters =
      published_acts_query()
      |> all_subdivisions_acts_query()

    special_subdivisions_acts_with_activities_counters =
      published_acts_query()
      |> join_act_subdivision(subdivision)

    full_query = union_all(all_subdvisions_acts_with_activity_counters, ^special_subdivisions_acts_with_activities_counters)

    act_ids = from(q in subquery(full_query, prefix: prefix), select: q.id) |> Repo.all(prefix: prefix)

    published_acts_query()
    |> get_items_by_ids(act_ids)
    |> activity_counters_query(prefix)
    |> order_acts_by_last_activity_date()
    |> Repo.all(prefix: prefix)
    |> Translator.translate(locale)
  end

  def join_activity_all_subdivisions_acts(queryable) do
    from activity in queryable,
      inner_join: a in Act, on: ((a.id == activity.act_id) and (a.all_subdivisions == ^true) and (a.draft == ^false))
  end

  def join_specail_subdivision_plus_all_subdivisions_activities(queryable, subdivision) do
    from activity in queryable,
      inner_join: a in Act, on: ((a.id == activity.act_id) and (a.draft == ^false)),
      left_join: asubd in ActSubdivision, on: (a.id == asubd.act_id),
      where: (asubd.subdivision_id == ^subdivision.id) or is_nil(asubd.subdivision_id)
  end


  @doc """
  Subdivision's activities list order by their date, the newest first.
  """
  def list_subdivision_activities(subdivision, locale) do
    prefix = Ecto.get_meta(subdivision, :prefix)


    published_activities_query()
    |> join_specail_subdivision_plus_all_subdivisions_activities(subdivision)
    |> Monitoring.activities_with_associations_query()
    |> Repo.all(prefix: prefix)
    |> Translator.translate(locale)
  end


  #
  #  ODS
  #
  def get_ods!(id, locale, prefix) do
    Ods
    |> Repo.get!(id, prefix: prefix)
    |> Translator.translate(locale)
  end


  def ods_act_activites_counter_query(prefix) do
    from ao in ActOds,
      join: act in Act, on: (ao.act_id == act.id and act.draft==^false),
      left_join: activity in Activity, on: (act.id == activity.act_id and activity.draft==^false),
      group_by: [ao.ods_id, ao.act_id],
      select: %{ods_id: ao.ods_id, act_id: ao.act_id, activities_count: count(activity.id)}
  end

  def join_act_ods(queryable, ods) do
    from act in queryable,
      inner_join: ao in ActOds, on: ((act.id == ao.act_id) and (ao.ods_id == ^ods.id))
  end

  def join_ods_activities(queriable, ods_id) do
    from activity in queriable,
      inner_join: ao in ActOds, on: (activity.act_id == ao.act_id) and (ao.ods_id == ^ods_id)
  end


  def list_ods_with_acts_and_activities_counter(locale, prefix) do
    q = from o in Ods,
      join: ods_act_activites_counter in subquery(ods_act_activites_counter_query(prefix), prefix: prefix),
      on: ods_act_activites_counter.ods_id == o.id,
      group_by: [o.id, ods_act_activites_counter.ods_id],
      select: {o, count(ods_act_activites_counter.act_id), sum(ods_act_activites_counter.activities_count)},
      order_by: o.position

    Repo.all(q, prefix: prefix)
    |> Translator.translate(locale)
  end

  def list_ods_acts_with_activity_counters(ods, locale) do
    prefix = Ecto.get_meta(ods, :prefix)

    published_acts_query()
    |> join_act_ods(ods)
    |> activity_counters_query(prefix)
    |> order_acts_by_last_activity_date()
    |> Repo.all(prefix: prefix)
    |> Translator.translate(locale)
  end

  @doc """
  ODS activities list order by thir date, the newest first.
  """
  def list_ods_activities(ods, locale) do
    ## OLD: does not respect the custom order in the preload
    ## ods_with_activities =
    ##   ods
    ##   |> Repo.preload([activities: Monitoring.activities_with_associations_query(published_activities_query())])

    ## ods_with_activities.activities |> Translator.translate(locale)

    published_activities_query()
    |> join_ods_activities(ods.id)
    |> Monitoring.activities_with_associations_query()
    |> Repo.all(prefix: Ecto.get_meta(ods, :prefix))
    |> Translator.translate(locale)
  end

  #
  # Acts
  #
  def get_act_with_activities_and_archive!(id, locale, prefix) do
    get_published_act!(id, prefix)
    |> Plan.preload_act_associations()
    |> Plan.preload_act_activities_and_archive(published_activities_query())
    |> Translator.translate(locale)
  end

  #
  # Activities
  #
  def list_activities_with_archive(locale, prefix) do
    list_activities_with_archive_query
    |> Repo.all(prefix: prefix)
    |> Translator.translate(locale)
  end

  def list_activities_with_archive_query() do
    published_activities_query
    |> Monitoring.activities_with_associations_query
  end

  #
  # Entity Services
  #
  def list_entity_services(locale, prefix) do
    from(s in EntityService, order_by: s.position)
    |> Repo.all(prefix: prefix)
    |> Translator.translate(locale)
  end
end
