defmodule OgpEuskadi.Plan do
  @moduledoc """
  The Plan context.
  """

  import Ecto.Query, warn: false
  import OgpEuskadi.QueryableTools

  alias OgpEuskadi.Repo

  alias OgpEuskadi.Plan.Goal
  alias OgpEuskadi.Plan.Act
  alias OgpEuskadi.Plan.ActTopic
  alias OgpEuskadi.Plan.ActDiscard
  alias OgpEuskadi.Monitoring
  alias OgpEuskadi.Monitoring.Activity
  alias OgpEuskadi.Monitoring.Indicator
  alias OgpEuskadi.Monitoring.ActivityIndicator

  alias OgpEuskadi.Basic.Ods
  alias OgpEuskadi.Basic.Topic

  @doc """
  Returns the list of goals.

  ## Examples

      iex> list_goals()
      [%Goal{}, ...]

  """
  def list_goals(prefix) do
    from(g in Goal, order_by: g.position)
    |> Repo.all(prefix: prefix)
  end

  def list_goals_with_actions(prefix) do
    from(g in Goal, order_by: g.position)
    |> Repo.all(prefix: prefix)
    |> Repo.preload([acts: from(a in Act, order_by: a.position)])
  end

  def list_goals_with_actions_and_activities(prefix) do
    list_goals_with_actions(prefix)
    |> Repo.preload(acts: :activities)
  end

  def list_goals_with_acts_counter(prefix) do
    q = from g in Goal,
        left_join: a in assoc(g, :acts),
        group_by: [g.id, a.goal_id],
        select: {g, count(a.goal_id)},
        order_by: g.position

    Repo.all(q, prefix: prefix)
  end

  @doc """
  Gets a single goal.

  Raises `Ecto.NoResultsError` if the Goal does not exist.

  ## Examples

      iex> get_goal!(123)
      %Goal{}

      iex> get_goal!(456)
      ** (Ecto.NoResultsError)

  """
  def get_goal!(id, prefix) do
    Goal
    |> Repo.get!(id, prefix: prefix)
    # |> Repo.preload(ods: from(o in Ods, order_by: o.position))
  end

  def goal_acts_query(goal) do
    from(act in Act, where: [goal_id: ^goal.id], order_by: act.position)
  end

  def list_goal_acts(goal) do
    goal_acts_query(goal)
    |> Repo.all(prefix: Ecto.get_meta(goal, :prefix))
  end

  def get_goal_with_acts!(id, prefix) do
    acts_q = from act in Act,
      left_join: activity in assoc(act, :activities),
      group_by: [act.id, activity.act_id],
      select: {act, count(activity.act_id)},
      order_by: act.position


    get_goal!(id, prefix)
    |> Repo.preload(acts: [:goal, :topics])
  end


  def acts_with_activity_counters_query(queryable) do
    from act in queryable,
      left_join: activity in assoc(act, :activities),
      group_by: [act.id, activity.act_id],
      select: {act, count(activity.act_id)},
      preload: [:goal, :topics, :subdivisions, :ods, indicators: :indicator_kind]
  end

  def get_goal_acts_with_activity_counters(goal) do
    goal_acts_query(goal)
    |> acts_with_activity_counters_query()
    |> Repo.all(prefix: Ecto.get_meta(goal, :prefix))
  end

  @doc """
  Creates a goal.

  ## Examples

      iex> create_goal(%{field: value})
      {:ok, %Goal{}}

      iex> create_goal(%{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def create_goal(attrs, prefix) do
    %Goal{}
    |> Goal.changeset(attrs)
    |> Repo.insert(prefix: prefix)
  end

  @doc """
  Updates a goal.

  ## Examples

      iex> update_goal(goal, %{field: new_value})
      {:ok, %Goal{}}

      iex> update_goal(goal, %{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def update_goal(%Goal{} = goal, attrs) do
    goal
    |> Goal.changeset(attrs)
    |> Repo.update(prefix: Ecto.get_meta(goal, :prefix))
  end

  @doc """
  Deletes a Goal.

  ## Examples

      iex> delete_goal(goal)
      {:ok, %Goal{}}

      iex> delete_goal(goal)
      {:error, %Ecto.Changeset{}}

  """
  def delete_goal(%Goal{} = goal) do
    goal
    |> Ecto.Changeset.change
    |> Ecto.Changeset.no_assoc_constraint(:acts)
    |> Repo.delete
  end

  @doc """
  Returns an `%Ecto.Changeset{}` for tracking goal changes.

  ## Examples

      iex> change_goal(goal)
      %Ecto.Changeset{source: %Goal{}}

  """
  def change_goal(%Goal{} = goal, attrs \\ %{}) do
    goal
    |> Goal.changeset(attrs)
  end


  @doc """
  Set new goal positions according to the order of the goals ids in the list ordered_goal_ids
  """
  def reorder_goals(ordered_goal_ids, prefix) when is_list(ordered_goal_ids) do
    Repo.transaction(fn ->
      ordered_goal_ids
      |> Enum.with_index()
      |> Enum.map(fn {goal_id, n} ->
        get_goal!(goal_id, prefix)
        |> change_goal(%{ position: n+1})
        |> Repo.update!
      end)
    end)
  end

  @doc """
  Returns the list of acts.

  ## Examples

      iex> list_acts()
      [%Goal{}, ...]

  """
  def list_acts(prefix) do
    from(a in Act, preload: [:goal, :topics, :subdivisions, :ods, indicators: :indicator_kind])
    |> Repo.all(prefix: prefix)
  end

  def list_acts_with_indicators(prefix) do
    q =
      from act in Act,
      distinct: true,
      join: indicator in assoc(act, :indicators),
      join: goal in assoc(act, :goal),
      select: %{act | goal_position: fragment("?", goal.position)},
      order_by: [goal.position, act.position]

    Repo.all(q, prefix: prefix)
    |> Repo.preload([:goal, indicators: from(i in Indicator, order_by: i.id)])
  end

  @doc """
  Returns the list of acts with activities counter.
  Acts are ordered by last_activity_date

  ## Examples

      iex> list_acts_with_activities_counter()
      [{%Act{}, counter}, ...]

  """
  def list_acts_with_activities_counter(prefix) do
    q = from(act in Act,
      left_join: activity in assoc(act, :activities),
      group_by: [act.id, activity.act_id],
      select: {act, count(activity.act_id)})
    |> order_acts_by_last_activity_date()

    Repo.all(q, prefix: prefix)
    |> Enum.map(fn {a, counter} -> {Repo.preload(a, [:goal, :topics, :subdivisions, :indicators, :ods]), counter} end)
  end

  @doc """
  Returns an `%Ecto.Changeset{}` for tracking act changes.

  ## Examples

      iex> change_act(act)
      %Ecto.Changeset{source: %Act{}}

  """
  def change_act(%Act{} = act, attrs \\ %{}) do
    act
    |> Act.changeset(attrs)
    |> Map.put(:topics_ids, Act.get_current_topics_ids(act))
    |> Map.put(:subdivisions_ids, Act.get_current_subdivision_ids(act))
    |> Map.put(:ods_ids, Act.get_current_ods_ids(act))
  end

  @doc """
  Creates a act.

  ## Examples

      iex> create_act(%{field: value})
      {:ok, %Act{}}

      iex> create_act(%{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def create_act(attrs, prefix) do
    attrs = Map.merge(%{"last_activity_date" => Date.utc_today, "topics_ids" => [], "subdivisions_ids" => [], "ods_ids" => [], "prefix" => prefix}, attrs)

    %Act{}
    |> Act.changeset(attrs)
    |> Repo.insert(prefix: prefix)
  end

  @doc """
  Gets a single act.

  Raises `Ecto.NoResultsError` if the Act does not exist.

  ## Examples

      iex> get_act(123)
      %Act{}

      iex> get_act(456)
      ** (Ecto.NoResultsError)

  """
  def get_act!(id, prefix) do
    Repo.get!(Act, id, prefix: prefix)
    |> preload_act_associations()
  end

  def get_act_with_indicators!(id, prefix) do
    get_act!(id, prefix)
    |> Repo.preload([indicators: {from(i in Indicator, order_by: i.id), [:indicator_kind]}])
  end

  def preload_act_associations(act) do
    Repo.preload(act, [:goal, :topics, :subdivisions, indicators: {from(i in Indicator, order_by: i.id), [:indicator_kind]}, ods: from(o in Ods, order_by: o.position)])
  end

  @doc """
  Gets a single act with preloaded activities and goal.

  Raises `Ecto.NoResultsError` if the Act does not exist.

  ## Examples

      iex> get_act_with_activities(123)
      %Act{}

      iex> get_act_with_activities(456)
      ** (Ecto.NoResultsError)

  """
  def get_act_with_activities!(id, prefix) do
    activities_query =
      from(a in Activity,
           order_by: [desc: a.date, desc: a.updated_at],
           preload: [
             activity_indicators: ^Monitoring.activity_indicators_query,
             act: :indicators
           ])

    get_act!(id, prefix)
    |> Repo.preload([activities: activities_query])
  end

  @doc """
  Gets a single act with preloaded activities with photos and documents and goal.

  Raises `Ecto.NoResultsError` if the Act does not exist.

  ## Examples

      iex> get_act_with_activities_and_archive(123)
      %Act{}

      iex> get_act_with_activities_and_archive(456)
      ** (Ecto.NoResultsError)

  """
  def get_act_with_activities_and_archive!(id, prefix) do
    get_act!(id, prefix)
    |> preload_act_activities_and_archive()
  end

  def preload_act_activities_and_archive(act, activities_queryable \\ Activity) do
    Repo.preload(act, [
      topics: [],
      subdivisions: [],
      indicators: :indicator_kind,
      activities: Monitoring.activities_with_associations_query(activities_queryable)
    ])

  end

  @doc """
  Updates a act.

  ## Examples

      iex> update_act(act, %{field: new_value})
      {:ok, %Goal{}}

      iex> update_act(act, %{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def update_act(%Act{} = act, attrs) do
    act
    |> Act.changeset(attrs)
    |> Repo.update(prefix: Ecto.get_meta(act, :prefix)) # NOTE: prefix is necesary for put_assoc(:topics) to use it.
  end

  @doc """
  Deletes an Act.

  ## Examples

      iex> delete_act(act)
      {:ok, %Act{}}

      iex> delete_act(act)
      {:error, %Ecto.Changeset{}}

  """
  def delete_act(%Act{} = act) do
    act
    |> Ecto.Changeset.change
    |> Ecto.Changeset.no_assoc_constraint(:activities)
    |> Repo.delete()
  end

  @doc """
  Prepare changeset to discard an Act.
  """
  def change_act_discard_activity(%Act{} = act) do
    Monitoring.change_activity(%Activity{}, %{act_id: act.id, discards_act: true, activity_indicators: act.indicators |> Enum.map(&(%{indicator_id: &1.id}))})
  end

  @doc """
  Creates discard activity and marks act as discarded
  """
  def discard_act(%Act{} = act, activity_params) do
    prefix = Ecto.get_meta(act, :prefix)

    case Monitoring.create_activity(
      activity_params |> Map.put("discards_act", true) |> Map.put("draft", false),
      prefix
    ) do
      {:ok, %{activity: activity}} ->
        ActDiscard
        |> Repo.get!(act.id, prefix: prefix)
        |> ActDiscard.changeset(%{"discarded" =>  true})
        |> Repo.update()

      {:error, :activity, %Ecto.Changeset{} = changeset, _} ->
        {:error, changeset}
    end
  end

  @doc """
  Set new acts positions according to the order of the goals ids in the list ordered_act_ids
  """
  def reorder_acts(ordered_act_ids, prefix) when is_list(ordered_act_ids) do
    Repo.transaction(fn ->
      ordered_act_ids
      |> Enum.with_index()
      |> Enum.map(fn {act_id, n} ->
        get_act!(act_id, prefix)
        |> Act.changeset(%{ "position" => n+1})
        |> Repo.update!
      end)
    end)
  end


end
