defmodule OgpEuskadi.Plan.Act do
  use Ecto.Schema
  use OgpEuskadi.Translator, translates: [:title, :description], translates_assoc: [:goal, :topics, :subdivisions, :activities, :indicators, :ods]

  import Ecto.Query, warn: false
  import Ecto.Changeset

  import OgpEuskadi.DateInput

  alias OgpEuskadi.Basic
  alias OgpEuskadi.Config
  alias OgpEuskadi.Plan
  alias OgpEuskadi.Monitoring

  alias OgpEuskadi.Repo

  schema "acts" do
    field :budget, :integer
    field :description_es, :string
    field :description_eu, :string
    field :ends_on, OgpEuskadi.TextDate
    field :ends_on_expected, OgpEuskadi.TextDate
    field :starts_on, OgpEuskadi.TextDate
    field :starts_on_expected, OgpEuskadi.TextDate
    field :title_es, :string
    field :title_eu, :string
    field :draft, :boolean
    field :discarded, :boolean
    field :position, :integer
    field :all_subdivisions, :boolean

    # Translator fields
    field :description, :string, virtual: true
    field :title, :string, virtual: true

    # Query fields
    field :goal_position, :integer, virtual: true # Used in query to order acts by goal.position

    field :last_activity_date, :date

    belongs_to :goal, Plan.Goal

    has_many :activities, Monitoring.Activity
    has_one :discards_act_activity, Monitoring.Activity, where: [discards_act: true]

    has_many :indicators, Monitoring.Indicator, on_replace: :delete #on_delete: :delete_all dos not addes prefix to indicators table???

    has_many :act_topics, Plan.ActTopic, on_replace: :delete
    #on_delete: :delete_all, on_replace: :delete

    field :topics_ids, {:array, :integer}, virtual: true
    many_to_many :topics, Basic.Topic, join_through: Plan.ActTopic, on_replace: :delete #, on_delete: :delete_all

    field :subdivisions_ids, {:array, :integer}, virtual: true
    many_to_many :subdivisions, Config.Subdivision, join_through: Plan.ActSubdivision, on_replace: :delete #, on_delete: :delete_all

    has_many :act_ods, Plan.ActOds, on_replace: :delete
    field :ods_ids, {:array, :integer}, virtual: true
    many_to_many :ods, Basic.Ods, join_through: Plan.ActOds, on_replace: :delete

    timestamps()
  end

  @doc false
  def changeset(act, attrs) do
    prefix = Ecto.get_meta(act, :prefix) || attrs["prefix"]

    changeset =
      act
      |> cast(attrs, [:goal_id, :title_es, :title_eu, :description_es, :description_eu, :starts_on_expected, :starts_on, :ends_on_expected, :ends_on, :budget, :draft, :position, :last_activity_date, :all_subdivisions, :topics_ids, :subdivisions_ids, :ods_ids])
      |> validate_required([:goal_id, :title_es, :description_es, :starts_on_expected])
      |> validate_length(:title_es, max: 255)
      |> validate_length(:title_eu, max: 255)
      |> validate_change(:topics_ids, fn(:topics_ids, topics_ids) ->
           if topics_ids == [] do
             [topics_ids: "can't be blank"]
           else
             []
           end
         end)
 
    # Assign topics
    new_topics_ids = get_change(changeset, :topics_ids)
    changeset =
      if !!new_topics_ids do
        changeset
        |> put_assoc(:topics, get_topics_by_ids(new_topics_ids, prefix))
     else
        changeset
      end

    # Assign subdivisions
    changeset =
      case fetch_field(changeset, :all_subdivisions) do
        {_status, value} ->
          if value == true do
            remove_individual_subdivisions(changeset)
          else
            subdivisions = get_subdivisions_by_ids(get_change(changeset, :subdivisions_ids), prefix)
            update_individual_subdivisions(changeset, subdivisions)
          end

        _ -> changeset
      end

    # Assigns ODS
    new_ods_ids = get_change(changeset, :ods_ids)
    changeset =
      if !!new_ods_ids do
        changeset
        |> put_assoc(:ods, get_ods_by_ids(get_field(changeset, :ods_ids), prefix))
      else
        changeset
      end
  end

  def format_text_dates_for_html(changeset) do
    changeset
    |> put_change(:starts_on_expected, date_input_format(get_field(changeset, :starts_on_expected)))
    |> put_change(:starts_on, date_input_format(get_field(changeset, :starts_on)))
    |> put_change(:ends_on_expected, date_input_format(get_field(changeset, :ends_on_expected)))
    |> put_change(:ends_on, date_input_format(get_field(changeset, :ends_on)))
  end

  def fill_form_data(changeset) do
    changeset
    |> Map.put(:topics_ids, get_change(changeset, :topics_ids))
    |> Map.put(:subdivisions_ids, get_change(changeset, :subdivisions_ids))
    |> Map.put(:ods_ids, get_change(changeset, :ods_ids))
  end

  def format_text_dates_for_db(act) do
    act
    |> Map.put(:starts_on, date_input_format(act.starts_on))
    |> Map.put(:starts_on_expected, date_input_format(act.starts_on))
    |> Map.put(:ends_on, date_input_format(act.ends_on))
    |> Map.put(:ends_on_expected, date_input_format(act.ends_on))
  end

  def get_topics_by_ids("", _prefix) do
    []
  end

  def get_topics_by_ids(topics_ids, prefix) do
    from(t in Basic.Topic, where: t.id in ^topics_ids) |> Repo.all(prefix: prefix)
  end

  def get_current_topics_ids(act) do
    act = Repo.preload(act, :topics)
    Enum.map(act.topics, fn t -> t.id end)
  end

  def get_subdivisions_by_ids(nil, _prefix) do
    []
  end

  def get_subdivisions_by_ids([], _prefix) do
    []
  end

  def get_subdivisions_by_ids(subdivisions_ids, prefix) do
    from(sd in Config.Subdivision, where: sd.id in ^subdivisions_ids)
    |> Repo.all(prefix: prefix)
  end

  def get_current_subdivision_ids(act) do
    act = act |> Repo.preload(:subdivisions)
    Enum.map(act.subdivisions, fn sd -> sd.id end)
  end

  defp remove_individual_subdivisions(changeset) do
    changeset
    |> put_change(:subdivisions_ids, [])
    |> put_assoc(:subdivisions, [])
  end

  defp update_individual_subdivisions(changeset, subdivisions) do
    changeset
    |> put_change(:subdivisions_ids, Enum.map(subdivisions, fn sd -> sd.id end))
    |> put_assoc(:subdivisions, subdivisions)

  end

  def get_current_ods_ids(act) do
    a = act
    |> Repo.preload(:act_ods)

    a.act_ods |> Enum.map(&(&1.ods_id))
  end

  defp get_ods_by_ids(nil, prefix) do
    []
  end

  defp get_ods_by_ids(ids, prefix) do
    from(ods in Basic.Ods, where: ods.id in ^ids) |> Repo.all(prefix: prefix)
  end

end
