# Based on https://hexdocs.pm/trans/Trans.Translator.html#content
# and https://github.com/mathieuprog/i18n_helpers

defmodule OgpEuskadi.Translator do
  defmacro __using__(opts) do
    quote do
      Module.put_attribute(__MODULE__, :translatable_fields, unquote(translatable_fields(opts)))
      Module.put_attribute(__MODULE__, :translatable_assoc, unquote(translatable_assoc(opts)))

      @spec __translator__(:fields) :: list(atom)
      def __translator__(:fields), do: @translatable_fields
      def __translator__(:assoc), do: @translatable_assoc
    end
  end

  @spec translate(list | struct, String.t() | atom) :: list | struct

  def translate([], _locale), do: []

  def translate([head | tail], locale) do
    [
      translate(head, locale)
      | translate(tail, locale)
    ]
  end

  def translate(%{__struct__: _struct_name} = struct, locale) do
    translate_struct(struct, locale)
  end

  def translate({%{__struct__: _struct_name} = struct, counter1}, locale) do
    {translate_struct(struct, locale), counter1}
  end

  def translate({%{__struct__: _struct_name} = struct, counter1, counter2}, locale) do
    {translate_struct(struct, locale), counter1, counter2}
  end

  @spec translate_struct(struct, String.t() | atom) :: any
  def translate_struct(%{__struct__: module} = item, locale) do
    fields_to_translate = module.__translator__(:fields)
    assocs_to_translate = module.__translator__(:assoc)

    item = Enum.reduce(fields_to_translate, item, fn field, updated_item ->
      struct(updated_item, [{field, translate_field(updated_item, field, locale)}])
    end)

    item = Enum.reduce(assocs_to_translate, item, fn field, updated_item ->
      %{^field => assoc} = item

      case Ecto.assoc_loaded?(assoc) do
        true ->
          struct(updated_item, [{field, translate(assoc, locale)}])
        _ ->
          updated_item
      end
    end)

    item
  end

  def translate_field(item, field, locale) do
    translated =
      case Map.fetch(item, String.to_atom("#{field}_#{locale}")) do
        {:ok, nil} -> fallback_value(item, field)
        {:ok, ""} -> fallback_value(item, field)
        {:ok, translated} -> translated
        :error ->
         raise "No translation for field #{inspect(field)} and locale #{inspect(locale)}"
      end
    translated
  end

  defp fallback_value(item, field) do
    default_locale = "es"
    case Map.fetch(item, String.to_atom("#{field}_#{default_locale}")) do
      {:ok, translated} -> translated

      :error -> nil
    end
  end

  defp translatable_fields(opts) do
    case Keyword.fetch(opts, :translates) do
      {:ok, fields} when is_list(fields) ->
        fields

      _ ->
        raise ArgumentError,
          message:
            "Translator requires a 'translates' option that contains the list of translatable fields names"
    end
  end

  defp translatable_assoc(opts) do
    case Keyword.fetch(opts, :translates_assoc) do
      {:ok, assoc} when is_list(assoc) ->
        assoc

      _ ->
        []
    end
  end

end
