defmodule OgpEuskadiWeb.Admin.ActivityController do
  use OgpEuskadiWeb, :controller
  use OgpEuskadiWeb.CurrentPrefixHelper

  alias OgpEuskadi.Translator
  alias OgpEuskadi.Paginator
  alias OgpEuskadi.Basic
  alias OgpEuskadi.Plan
  alias OgpEuskadi.Monitoring
  alias OgpEuskadi.Monitoring.Activity
  alias OgpEuskadi.Monitoring.Indicator
  alias OgpEuskadi.Monitoring.ActivityIndicator

  def index(conn, params, prefix) do
    page_number = String.to_integer(Map.get(params, "page", "1"))

    paginated_activities = Monitoring.list_activities_query |> Paginator.new(%{page_number: page_number}, prefix)
    activities = paginated_activities.entries |> Translator.translate(conn.assigns.locale)
    pagination_data =
      if paginated_activities.last > 0 do
        Numerator.build(paginated_activities, show_first: true, show_last: true)
      else
        []
      end

    render(conn, "index.html", activities: activities, pagination_data: pagination_data)
  end

  def new(conn, params, prefix) do
    act_id = params["act_id"]
    changeset = Monitoring.change_activity(%Activity{}, %{act_id: act_id, activity_indicators: [%{indicator_id: 1}]})

    acts = Plan.list_acts_with_indicators(prefix) |> Translator.translate(conn.assigns.locale)

    indicator_kinds = Basic.list_indicator_kinds(prefix) |> Translator.translate(conn.assigns.locale)
    form_action = Routes.admin_activity_path(conn, :create, conn.assigns.tenant, conn.assigns.locale)

    render(conn, "new.html", changeset: changeset, form_action: form_action, acts: acts, indicator_kinds: indicator_kinds)
  end

  def create(conn, %{"activity" => activity_params}, prefix) do
    case Monitoring.create_activity(activity_params, prefix) do
      {:ok, %{activity: activity}} ->
        activity = Monitoring.preload_activity_act(activity)
        conn
        |> put_flash(:info, gettext("¡Actividad creada!"))
        |> redirect(to: Routes.admin_act_path(conn, :show, conn.assigns.tenant, conn.assigns.locale, activity.act))

      {:error, :activity, %Ecto.Changeset{} = changeset, _} ->
        changeset = changeset |> Activity.format_text_dates_for_html
        acts = Plan.list_acts(prefix) |> Translator.translate(conn.assigns.locale)
        indicator_kinds = Basic.list_indicator_kinds(prefix) |> Translator.translate(conn.assigns.locale)
        form_action = Routes.admin_activity_path(conn, :create, conn.assigns.tenant, conn.assigns.locale)
        render(conn, "new.html", activity: %Activity{}, changeset: changeset, form_action: form_action, acts: acts, indicator_kinds: indicator_kinds)
    end
  end

  def show(conn, %{"id" => id}, prefix) do
    activity = Monitoring.get_activity_with_archive!(id, prefix) |> Translator.translate(conn.assigns.locale)
    render(conn, "show.html", activity: activity)
  end

  def edit(conn, %{"id" => id}, prefix) do
    activity = Monitoring.get_activity!(id, prefix)

    changeset = Monitoring.change_activity(activity) |> Activity.format_text_dates_for_html
    acts = Plan.list_acts_with_indicators(prefix) |> Translator.translate(conn.assigns.locale)
    selected_act_id = Ecto.Changeset.get_change(changeset, :act_id) || activity.act_id
    indicator_kinds = Basic.list_indicator_kinds(prefix) |> Translator.translate(conn.assigns.locale)
    form_action = Routes.admin_activity_path(conn, :update, conn.assigns.tenant, conn.assigns.locale, activity)

    render(conn, "edit.html", activity: activity, changeset: changeset, form_action: form_action, acts: acts, indicator_kinds: indicator_kinds)
  end

  def update(conn, %{"id" => id, "activity" => activity_params}, prefix) do
    activity = Monitoring.get_activity!(id, prefix)

    case Monitoring.update_activity(activity, activity_params) do
      {:ok, %{activity: activity, activity_indicators: indicators}} ->
        conn
        |> put_flash(:info, gettext("¡Actividad actualizada!"))
        |> redirect(to: Routes.admin_activity_path(conn, :show, conn.assigns.tenant, conn.assigns.locale, activity))

      {:error, :activity, %Ecto.Changeset{} = changeset, _} ->
        changeset = changeset |> Activity.format_text_dates_for_html
        acts = Plan.list_acts(prefix) |> Translator.translate(conn.assigns.locale)
        selected_act_id = Ecto.Changeset.get_field(changeset, :act_id)
        indicator_kinds = Basic.list_indicator_kinds(prefix) |> Translator.translate(conn.assigns.locale)
        form_action = Routes.admin_activity_path(conn, :update, conn.assigns.tenant, conn.assigns.locale, activity)
        render(conn, "edit.html", activity: activity, changeset: changeset, form_action: form_action, acts: acts, indicator_kinds: indicator_kinds)
    end
  end

  def delete(conn, %{"id" => id}, prefix) do
    activity = Monitoring.get_activity!(id, prefix)
    {:ok, _activity} = Monitoring.delete_activity(activity)

    conn
    |> put_flash(:info, gettext("¡Actividad eliminada!"))
    |> redirect(to: Routes.admin_act_path(conn, :show, conn.assigns.tenant, conn.assigns.locale, activity.act))
  end


  defp get_indicators(acts, selected_act_id) do
    case acts |> Enum.filter(fn a -> a.id == selected_act_id end) do
      [selected_act] -> selected_act.indicators
      [] -> []
    end
  end
end
