# Source form set_locale plug https://github.com/smeevil/set_locale
# with modified determine_locale to redirect to the default locale taken from the config.
defmodule OgpEuskadiWeb.Plugs.SetLocale do
  import Plug.Conn

  defmodule Config do
    @enforce_keys [:gettext, :default_locale]
    defstruct [:gettext, :default_locale]
  end

  def init(opts) when is_tuple(hd(opts)), do: struct!(Config, opts)

  def call(
        %{
          request_path: request_path,
          params: %{
            "locale" => requested_locale
          }
        } = conn,
        config
  ) do
    if request_path != "/" and supported_locale?(requested_locale, config) do
      Gettext.put_locale(config.gettext, requested_locale)
      assign(conn, :locale, requested_locale)
    else
      path = rewrite_path(conn, requested_locale, config)

      conn
      |> redirect_to(path)
      |> halt
    end
  end

  def call(conn, config) do
    if !!conn.assigns[:tenant] do
      path = rewrite_path(conn, nil, config)

      conn
      |> redirect_to(path)
      |> halt
    else
      conn
    end
  end

  defp rewrite_path(%{request_path: request_path} = conn, requested_locale, config) do
    locale = determine_locale(conn, requested_locale, config)

    request_path
    |> maybe_strip_unsupported_locale
    |> localize_path(locale)
  end

  defp determine_locale(conn, nil, config) do
    config.default_locale
  end

  defp determine_locale(conn, requested_locale, config) do
    base = hd(String.split(requested_locale, "-"))

    if is_locale?(requested_locale) and supported_locale?(base, config) do
      base
    else
      determine_locale(conn, nil, config)
    end
  end

  defp supported_locales(config),
    do: Gettext.known_locales(config.gettext)

  defp maybe_strip_unsupported_locale(request_path) do
    maybe_locale = maybe_extract_locale(request_path)
    if maybe_locale, do: strip_unsupported_locale(request_path), else: request_path
  end

  defp maybe_extract_locale(request_path) when is_binary(request_path) do
    case String.split(request_path, "/") do
      [_, _tenant, maybe_locale | _] ->
        if is_locale?(maybe_locale), do: maybe_locale, else: nil
      _ ->
        nil
    end
  end

  defp maybe_extract_locale(_), do: nil

  defp is_locale?(maybe_locale), do: Regex.match?(~r/^[a-z]{2}(-[a-z]{2})?$/, maybe_locale)

  defp strip_unsupported_locale(request_path) do
    [_, tenant, _ | rest] = String.split(request_path, "/")
    "/#{tenant}" <> Enum.join(rest, "/")
  end

  # defp localize_path("/", locale), do: "/#{locale}"
  defp localize_path(request_path, locale) do
    [_, tenant |rest] = String.split(request_path, "/")
    "/#{tenant}/#{locale}/" <> Enum.join(rest, "/")
  end

  defp redirect_to(conn, path) do
    path = get_redirect_path(conn, path)
    Phoenix.Controller.redirect(conn, to: path)
  end

  defp get_redirect_path(%{query_string: query_string}, path) when query_string != "",
    do: path <> "?#{query_string}"

  defp get_redirect_path(_conn, path), do: path

  defp supported_locale?(locale, config), do: Enum.member?(supported_locales(config), locale)
end
