defmodule OgpEuskadiWeb.Router do
  import Phoenix.LiveDashboard.Router

  use OgpEuskadiWeb, :router
  use OgpEuskadiWeb.Plugs.ExceptionNotifier

  if Mix.env == :dev do
    # LiveDashboard config, see https://hexdocs.pm/phoenix_live_dashboard/Phoenix.LiveDashboard.html
    scope "/" do
      pipe_through [:browser]
      live_dashboard "/dashboard", metrics: OgpEuskadiWeb.Telemetry
    end

    forward "/sent_emails", Bamboo.SentEmailViewerPlug
  end

  pipeline :browser do
    plug :accepts, ["html"]
    plug :fetch_session
    plug :fetch_live_flash
    plug :protect_from_forgery
    plug :put_secure_browser_headers
    plug Triplex.ParamPlug, endpoint: OgpEuskadiWeb.Endpoint, param: :tenant, assign: :tenant
    plug OgpEuskadiWeb.Plugs.SetLocale, gettext: OgpEuskadiWeb.Gettext, default_locale: "es"
    plug OgpEuskadi.Plug.LoadConfig
  end

  pipeline :api do
    plug :accepts, ["json"]
  end

  pipeline :jwt_authenticated do
    plug OgpEuskadi.Plugs.ApiAuthPipeline
    plug OgpEuskadiWeb.Plugs.ApiSetPrefix
  end

  pipeline :private do
    plug OgpEuskadiWeb.Auth
    plug :authenticate_user
    plug :put_layout, {OgpEuskadiWeb.LayoutView, "admin.html"}
  end

  pipeline :tenant do
    plug OgpEuskadiWeb.Auth
    plug :put_layout, {OgpEuskadiWeb.LayoutView, "tenant.html"}
  end

  # Site index
  scope "/", OgpEuskadiWeb do
    pipe_through :browser
    # you need this entry to support the default root without tenant and locale
    get "/", SiteController, :home
  end

  scope "/api/v1", OgpEuskadiWeb.Api, as: :api do
    pipe_through :api

    post "/auth/access_token", AuthController, :access_token
  end

  scope "/api/v1", OgpEuskadiWeb.Api, as: :api do
    pipe_through [:api, :jwt_authenticated]

    resources "/topics", TopicController, only: [:index]
    resources "/ods", OdsController, only: [:index]
    resources "/subdivisions", SubdivisionController, only: [:index]
    resources "/indicator_kinds", IndicatorKindController, only: [:index]

    resources "/goals", GoalController, except: [:new, :edit]
    resources "/acts", ActController, except: [:new, :edit] do
      resources "/indicators", IndicatorController, only: [:index]
      resources "/activities", ActivityController, only: [:index]
    end
    resources "/indicators", IndicatorController, only: [:show, :create, :update, :delete]
    resources "/activities", ActivityController, only: [:show, :create, :update, :delete]
  end

  scope "/:locale", OgpEuskadiWeb do
    pipe_through :browser
    # you need this entry to support the default root without tenant but with locale
    get "/", SiteController, :index
  end

  scope "/site", OgpEuskadiWeb do
    pipe_through :browser
    # you need this entry to support the default root without locale, it will never be called
    get "/", SiteController, :dummy
  end

  scope "/site/:locale", OgpEuskadiWeb do
    pipe_through :browser
    get "/", SiteController, :index
  end

  scope "/:tenant", OgpEuskadiWeb do
    pipe_through :browser
    # you need this entry to support the default root without locale, it will never be called
    get "/", SiteController, :dummy
  end

  scope "/:tenant/:locale", OgpEuskadiWeb do
    pipe_through [:browser, :tenant]

    get "/", PlanController, :index
    resources "/sessions", SessionController, only: [:new, :create, :delete]
    resources "/goal", GoalController, only: [:show]
    resources "/act", ActController, only: [:show]
    resources "/activity", ActivityController, only: [:index]
    get "/topic/:id/activities", TopicController, :show_activities
    get "/topic/:id/acts", TopicController, :show_acts
    resources "/topic", TopicController, only: [:index, :show]
    get "/subdivision/:id/activities", SubdivisionController, :show_activities
    get "/subdivision/:id/acts", SubdivisionController, :show_acts
    resources "/subdivision", SubdivisionController, only: [:index, :show]
    resources "/ods", OdsController, only: [:index, :show]
    get "/ods/:id/activities", OdsController, :show_activities
    get "/ods/:id/acts", OdsController, :show_acts

    resources "/entity_service", EntityServiceController, only: [:index]

    resources "/password_reset", PasswordResetController, only: [:new, :create, :edit, :update], singleton: true

    get "/search", SearchController, :index
  end


  scope "/:tenant/:locale/admin", OgpEuskadiWeb.Admin, as: :admin do
    pipe_through [:browser, :private]

    resources "/act", ActController do
      resources "/indicator", IndicatorController
      resources "/discard", ActDiscardController, singleton: true, as: :discard
    end

    resources "/activity", ActivityController
    get "/goal/sort", GoalController, :sort, as: :sort_goals
    post "/goal/save_order", GoalController, :save_order, as: :save_goals_order
    resources "/goal", GoalController do
      get "/act/sort", ActController, :sort, as: :sort_acts
      post "/act/save_order", ActController, :save_order, as: :save_acts_order
    end
    resources "/photo", PhotoController, except: [:index]
    resources "/document", DocumentController, except: [:index]
    resources "/plan", PlanController, only: [:index]

    resources "/users", UserController
    resources "/subdivision", SubdivisionController, except: [:index]
    resources "/entity_service", EntityServiceController, except: [:index]
    resources "/config", ConfigController, only: [:show, :edit, :create, :update], singleton: true

    resources "/account", AccountController, only: [:edit, :update], singleton: true

    get "/", PlanController, :index
  end
end
