# Reference: http://blog.plataformatec.com.br/2016/09/dynamic-forms-with-phoenix/
defmodule OgpEuskadiWeb.InputHelpers do
  use Phoenix.HTML

  def input(form, field, opts \\ %{}) do
    type = opts[:using] || Phoenix.HTML.Form.input_type(form, field)

    wrapper_opts = [class: "#{wrapper_class(type)} #{state_class(form, field)} #{validations_class(form, field)}"]
    label_opts = [class: label_class(type)]
    input_html_opts = opts[:html] || []
    input_opts = Enum.concat([class: input_class(type)], input_html_opts)
    options = opts[:options] || []

    content_tag :div, wrapper_opts do
      label = label(form, field, opts[:label] || humanize(field), label_opts)
      input =
        case type do
          :select -> apply(Phoenix.HTML.Form, type, [form, field, options, input_opts])
          _ -> apply(Phoenix.HTML.Form, type, [form, field, input_opts])
        end
      help_text =
        if !!opts[:help_text] do
          content_tag(:small, opts[:help_text], class: "text-muted input-help-text clearfix")
        else
          ""
        end
      error = OgpEuskadiWeb.ErrorHelpers.error_tag(form, field) || ""

      case type do
        :checkbox -> [input, label, error]
        _ -> [label, input, help_text, error]
      end
    end
  end

  def translatable_input(form, field, opts \\ %{}) do
    type = opts[:using] || Phoenix.HTML.Form.input_type(form, field)

    validations_field = String.to_atom("#{field}_es")
    wrapper_opts = [class: "#{wrapper_class(type)} #{state_class(form, validations_field)} #{validations_class(form, validations_field)}"]
    label_opts = [class: label_class(type)]
    input_opts = [class: "form-control"]

    content_tag :div, wrapper_opts do
      label = label(form, field, opts[:label] || humanize(field), label_opts)

      input_group_special_class = opts[:html][:input_group_class] || ""
      inputs =
        ["es", "eu"]
        |> Enum.map(fn locale ->
          localized_field = String.to_atom("#{field}_#{locale}")
          prefix = content_tag(:div, class: "input-group-prepend") do
            content_tag(:span, locale, class: "input-group-text")
          end
          input = apply(Phoenix.HTML.Form, type, [form, localized_field, input_opts])
          help_text =
            if !!opts[:help_text] do
              content_tag(:small, opts[:help_text][String.to_atom(locale)], class: "text-muted input-help-text clearfix")
            else
              ""
            end
          error = OgpEuskadiWeb.ErrorHelpers.error_tag(form, localized_field) || ""

          {:safe, content} = content_tag :div, class: "mb-2" do
            [content_tag(:div, [prefix, input], class: "input-group #{input_group_special_class}"), help_text, error]
          end
          content
          end)
        |> Enum.join()

      [label, {:safe, inputs}]
    end
  end

  def multiple_checkbox(form, field, options, checked_options \\ [], opts \\ %{}) do
    options
    |> Enum.map(fn {value, key} -> checkbox_with_label(form, field, key, value, Enum.member?(checked_options, key)) end)
  end

  # Inspired by https://elixirforum.com/t/many-to-many-checkbox-form/17216/6?
  defp checkbox_with_label(form, field, key, value, checked) do
    field_id = input_id(form, field, key)

    checkbox = apply(Phoenix.HTML.Form, :checkbox, [form, field, [checked: checked, checked_value: key, unchecked_value: "", hidden_input: false, id: field_id, class: "form-check-input", name: input_name(form, field) <> "[]"]])
    label = label(form, field, value, class: "form-check-label", for: field_id)

    content_tag :li, class: "form-check" do
      [checkbox, label]
    end
  end

  def cancel(label, opts \\ %{}) do
    link label, to: opts[:url] || "#", class: "btn btn-outline-secondary ml-2"
  end

  def state_class(form, field) do
    cond do
      !form.source.action -> ""
      form.errors[field] -> "invalid"
      true -> "valid"
    end
  end

  def validations_class(form, field) do
    validations = Phoenix.HTML.Form.input_validations(form, field)
    # To use the browser validations, add validations as input options:
    # input_opts = Keyword.merge(validations, input_opts)

    cond do
      Keyword.has_key?(validations, :required) && Keyword.get(validations, :required) ->
        "required"
      true ->
        "optional"
    end
  end


  defp wrapper_class(input_type) do
    case input_type do
      :checkbox -> "form-check"
      _ -> "form-group"
    end
  end

  defp label_class(input_type) do
    case input_type do
      :checkbox -> "form-check-label"
      _ -> "control-label"
    end
  end

  defp input_class(input_type) do
    case input_type do
      :checkbox -> "form-check-input"
      _ -> "form-control"
    end
  end

  def input_state_class(form, field) do
    cond do
      !form.source.action -> ""
      form.errors[field] -> "invalid"
      true -> "valid"
    end
  end
end
