defmodule OgpEuskadi.ArchiveTest do
  use OgpEuskadi.DataCase

  alias OgpEuskadi.Archive
  alias OgpEuskadi.Accounts

  def fixture(:user) do
    {:ok, user} = Accounts.create_user(%{username: "Test", credential: %{email: "test@efaber.net", password: "secret"}}, "ogpe_test_tenant")
    Accounts.get_user!(user.id, "ogpe_test_tenant")
  end

  describe "photos" do
    alias OgpEuskadi.Archive.Photo

    setup [:create_user_and_activity]

    @valid_attrs %{caption_es: "some caption", image:  %Plug.Upload{path: "test/assets/test.png", filename: "test.png"}}
    @update_attrs %{caption_es: "new caption"}
    @invalid_attrs %{caption_es: nil, image: nil}

    def photo_fixture(user, attrs \\ %{}) do
      attrs = attrs |> Enum.into(@valid_attrs)
      {:ok, photo} = Archive.create_photo(user, attrs, "ogpe_test_tenant")

      Archive.get_photo!(photo.id, "ogpe_test_tenant")
    end

    test "list_photos/0 returns all photos", %{user: user, activity: activity} do
      photo = photo_fixture(user, %{activity_id: activity.id})
      assert Archive.list_photos("ogpe_test_tenant") |> Enum.map(fn d -> d.id end) == [photo.id]
    end

    test "get_photo!/1 returns the photo with given id", %{user: user, activity: activity} do
      photo = photo_fixture(user, %{activity_id: activity.id})
      assert Archive.get_photo!(photo.id, "ogpe_test_tenant") == photo
    end

    test "create_photo/1 with valid data creates a photo",  %{user: user, activity: activity} do
      assert {:ok, %Photo{} = photo} = Archive.create_photo(user, %{activity_id: activity.id} |> Enum.into(@valid_attrs), "ogpe_test_tenant")
      assert photo.activity_id == activity.id
      assert photo.image.file_name == "test.png"
    end

    test "create_photo/1 with invalid data returns error changeset",  %{user: user} do
      assert {:error, %Ecto.Changeset{}} = Archive.create_photo(user, @invalid_attrs, "ogpe_test_tenant")
    end

    test "update_photo/2 with valid data updates the photo",  %{user: user, activity: activity} do
      photo = photo_fixture(user, %{activity_id: activity.id})
      assert {:ok, %Photo{} = photo} = Archive.update_photo(user, photo, @update_attrs)
      assert photo.activity_id == activity.id
      assert photo.caption_es == "new caption"
    end

    test "update_photo/2 with invalid data returns error changeset",  %{user: user, activity: activity} do
      photo = photo_fixture(user,  %{activity_id: activity.id})
      assert {:error, %Ecto.Changeset{}} = Archive.update_photo(user, photo, @invalid_attrs)
      assert photo == Archive.get_photo!(photo.id, "ogpe_test_tenant")
    end

    test "delete_photo/1 deletes the photo", %{user: user, activity: activity} do
      photo = photo_fixture(user, %{activity_id: activity.id})
      assert {:ok, %Photo{}} = Archive.delete_photo(photo)
      assert_raise Ecto.NoResultsError, fn -> Archive.get_photo!(photo.id, "ogpe_test_tenant") end
    end

    test "change_photo/1 returns a photo changeset",  %{user: user, activity: activity} do
      photo = photo_fixture(user, %{activity_id: activity.id})
      assert %Ecto.Changeset{} = Archive.change_photo(photo)
    end
  end

  describe "documents" do
    alias OgpEuskadi.Archive.Document

    setup [:create_user_and_activity]

    @valid_attrs %{caption_es: "some caption", url_es: "test.com/document.txt" }
    @valid_file_upload_attrs %{caption_es: "some caption", file:  %Plug.Upload{path: "test/assets/test.txt", filename: "test.txt"}}
    @update_attrs %{caption_es: "some updated caption", url_es: "test.com/other_document.txt"}
    @invalid_attrs %{caption_es: "Caption", file: nil, url_es: nil}

    def document_fixture(user, attrs \\ %{}) do
      attrs = attrs |> Enum.into(@valid_attrs)
      {:ok, document} = Archive.create_document(user, attrs, "ogpe_test_tenant")

      Archive.get_document!(document.id, "ogpe_test_tenant")
    end

    def document_with_file_fixture(user, attrs \\ %{}) do
      attrs = attrs |> Enum.into(@valid_file_upload_attrs)
      {:ok, document} = Archive.create_document(user, attrs, "ogpe_test_tenant")

      Archive.get_document!(document.id, "ogpe_test_tenant")
    end

    test "list_documents/0 returns all documents",  %{user: user, activity: activity} do
      document = document_fixture(user, %{activity_id: activity.id})
      assert Archive.list_documents("ogpe_test_tenant") |> Enum.map(fn d -> d.id end) == [document.id]
    end

    test "get_document!/1 returns the document with given id",  %{user: user, activity: activity} do
      document = document_fixture(user, %{activity_id: activity.id})
      assert Archive.get_document!(document.id, "ogpe_test_tenant") == document
    end

    test "create_document/1 with file upload creates a document",  %{user: user, activity: activity} do
      assert {:ok, %Document{} = document} = Archive.create_document(user, %{activity_id: activity.id} |> Enum.into(@valid_file_upload_attrs), "ogpe_test_tenant")

      assert document.activity_id == activity.id
      assert document.caption_es == "some caption"
      assert document.file.file_name == "test.txt"
      assert document.user_id == user.id
    end

    test "create_document/1 with url creates a document",  %{user: user, activity: activity} do
      assert {:ok, %Document{} = document} = Archive.create_document(user, %{activity_id: activity.id} |> Enum.into(@valid_attrs), "ogpe_test_tenant")

      assert document.activity_id == activity.id
      assert document.caption_es == "some caption"
      assert is_nil(document.file)
      assert document.user_id == user.id
    end

    test "create_document/1 with url and file returns error changeset",  %{user: user, activity: activity} do
      assert {:error, %Ecto.Changeset{}=changeset} = Archive.create_document(user, %{activity_id: activity.id, url_es: "http://test.com/document.txt"} |> Enum.into(@valid_file_upload_attrs), "ogpe_test_tenant")
    end

    test "create_document/1 with invalid data returns error changeset",  %{user: user, activity: activity} do
      assert {:error, %Ecto.Changeset{}=changeset} = Archive.create_document(user, %{activity_id: activity.id} |> Enum.into(@invalid_attrs), "ogpe_test_tenant")
    end

    test "update_document/2 with caption attribute updates the document caption",  %{user: user, activity: activity} do
      document = document_fixture(user, %{activity_id: activity.id})
      assert {:ok, %Document{} = document} = Archive.update_document(user, document, %{caption_es: "New caption"})
      assert document.caption_es == "New caption"
    end

    test "update_document/2 replaces file with url when remove_file attribute is true", %{user: user, activity: activity} do
      document = document_with_file_fixture(user, %{activity_id: activity.id})
      assert {:ok, %Document{} = document} = Archive.update_document(user, document, %{"url_es" => "http://test.com/new_document.txt", "remove_file" => true})
      assert is_nil(document.file)
      assert document.url_es == "http://test.com/new_document.txt"
    end

    test "update_document/2 with valid data updates the document",  %{user: user, activity: activity} do
      document = document_fixture(user, %{activity_id: activity.id})
      assert {:ok, %Document{} = document} = Archive.update_document(user, document, @update_attrs)
      assert document.caption_es == "some updated caption"
    end

    test "update_document/2 with file and url returns error changeset",  %{user: user, activity: activity} do
      document = document_with_file_fixture(user, %{activity_id: activity.id})
      assert {:error, %Ecto.Changeset{} = changeset} = Archive.update_document(user, document, %{"url_es" => "http://test.com/new_document.txt", "remove_file" => "false"})
      assert Keyword.fetch(changeset.errors, :file) == :error # no errors on file.
      assert Keyword.fetch(changeset.errors, :url_es) == :error # no errors on url_es
      {:ok, _remove_file_errors} = Keyword.fetch(changeset.errors, :remove_file)

      assert document == Archive.get_document!(document.id, "ogpe_test_tenant")
      assert is_nil(document.url_es)
    end


    test "update_document/2 with invalid data returns error changeset",  %{user: user, activity: activity} do
      document = document_fixture(user, %{activity_id: activity.id})
      assert {:error, %Ecto.Changeset{}} = Archive.update_document(user, document, @invalid_attrs)
      assert document == Archive.get_document!(document.id, "ogpe_test_tenant")
    end

    test "delete_document/1 deletes the document",  %{user: user, activity: activity} do
      document = document_fixture(user, %{activity_id: activity.id})
      assert {:ok, %Document{}} = Archive.delete_document(document)
      assert_raise Ecto.NoResultsError, fn -> Archive.get_document!(document.id, "ogpe_test_tenant") end
    end

    test "change_document/1 returns a document changeset",  %{user: user, activity: activity} do
      document = document_fixture(user, %{activity_id: activity.id})
      assert %Ecto.Changeset{} = Archive.change_document(document)
    end
  end

  defp create_user_and_activity(_) do
    user = fixture(:user)
    act = tenant_insert(:act)
    activity = add_activity(act) |> tenant_insert()

    {:ok, user: user, activity: activity}
  end
end
