defmodule OgpEuskadi.MonitoringTest do
  use OgpEuskadi.DataCase

  alias OgpEuskadi.Repo

  alias OgpEuskadi.Monitoring
  alias OgpEuskadi.Plan

  describe "activities" do
    alias OgpEuskadi.Monitoring.Activity

    @prefix "ogpe_test_tenant"

    @valid_attrs %{"date" => ~D[2010-04-17], "description_es" => "some description", "title_es" => "some title"}
    @update_attrs %{"date" => ~D[2011-05-18], "description_es" => "some updated description", "title_es" => "some updated title"}
    @invalid_attrs %{"date" => nil, "description_es" => nil, "title_es" => nil}

    def activity_fixture(attrs \\ %{}) do
      act = attrs["act"] || tenant_insert(:act)
      [indicator | _tail] = act.indicators

      attrs =
        attrs
        |> Map.put("act_id", act.id)
        |> Map.put("activity_indicators", %{
          "0" => %{
            "indicator_id" => indicator.id,
            "indicator_value" => "40"
          }
        })

      {:ok, %{activity: activity}} =
        attrs
        |> Enum.into(@valid_attrs)
        |> Monitoring.create_activity(@prefix)

      activity


      # Change fixture element values to match the DB values
      db_date = "#{String.pad_leading(Integer.to_string(activity.date.day), 2, "0")}/#{String.pad_leading(Integer.to_string(activity.date.month), 2, "0")}/#{activity.date.year}"
      activity =
        activity
        |> Map.put(:date, db_date)        # DB value has different format from the initial value
        |> Map.put(:discards_act, false)  # By default discards_act is false
        |> Map.put(:draft, true)          # By default draft is true

    end

    test "list_activities/0 returns all activities" do
      activity =
        activity_fixture() |> Repo.preload([activity_indicators: [indicator: :indicator_kind], act: :goal])

      assert Monitoring.list_activities(@prefix) == [activity]
    end

    test "get_activity!/1 returns the activity with given id" do
      activity =
        activity_fixture() |> Repo.preload([activity_indicators: [indicator: :indicator_kind], act: :goal])

      assert Monitoring.get_activity!(activity.id, @prefix) == activity
    end

    test "create_activity/2 with valid data creates activity" do
      act = tenant_insert(:act)
      [indicator | _tail] = act.indicators

      all_attrs =
        @valid_attrs
        |> Map.put("act_id", act.id)
        |> Map.put("activity_indicators", %{
          "0" => %{
            "indicator_id" => indicator.id,
            "indicator_value" => "40"
          }
        })

      assert {:ok, %{activity: %Activity{} = activity}} = Monitoring.create_activity(all_attrs, @prefix)

      # Check activitity is created
      assert activity.act_id == act.id
      assert activity.date == ~D[2010-04-17]
      assert activity.description_es == "some description"
      assert activity.title_es == "some title"
    end

    test "create_activity/2 with valid data and draft set to false creates activity but does not update act and indicator value" do
      act = tenant_insert(:act)
      [indicator | _tail] = act.indicators

      all_attrs =
        @valid_attrs
        |> Map.put("draft", true)
        |> Map.put("act_id", act.id)
        |> Map.put("activity_indicators", %{
          "0" => %{
            "indicator_id" => indicator.id,
            "indicator_value" => "40"
          }
        })

      assert {:ok, %{activity: %Activity{} = activity}} = Monitoring.create_activity(all_attrs, @prefix)

      # Check activitity is created
      assert activity.draft == true

      # Check act last_activity_date is not updated
      act = Plan.get_act!(act.id, @prefix)
      assert act.last_activity_date != activity.date

      # Check indicator current value is not 40
      indicator = Monitoring.get_indicator!(indicator.id, @prefix)
      assert indicator.current_value != 40
    end

    test "create_activity/2 with valid data and draft set to false creates activity and updates act and indicator value" do
      act = tenant_insert(:act)
      [indicator | _tail] = act.indicators

      all_attrs =
        @valid_attrs
        |> Map.put("draft", false)
        |> Map.put("act_id", act.id)
        |> Map.put("activity_indicators", %{
          "0" => %{
            "indicator_id" => indicator.id,
            "indicator_value" => "40"
          }
        })

      assert {:ok, %{activity: %Activity{} = activity}} = Monitoring.create_activity(all_attrs, @prefix)

      # Check activitity is created
      assert activity.draft == false

      # Check act last_activity_date is updated
      act = Plan.get_act!(act.id, @prefix)
      assert act.last_activity_date == activity.date

      # Check indicator current value is set to 40
      indicator = Monitoring.get_indicator!(indicator.id, @prefix)
      assert Decimal.to_integer(indicator.current_value) == 40
    end

    test "create_activity/1 with invalid data returns error changeset" do
      assert {:error, :activity, %Ecto.Changeset{}, _} = Monitoring.create_activity(@invalid_attrs)
    end

    test "update_activity/2 with valid data updates the activity" do
      activity = activity_fixture()

      assert {:ok, %{activity: %Activity{} = activity}} = Monitoring.update_activity(activity, @update_attrs)
      assert activity.date == ~D[2011-05-18]
      assert activity.description_es == "some updated description"
      assert activity.title_es == "some updated title"
    end

    test "update_activity/2 with valid data and activty in draft does not udate act not indicator value" do
      act = tenant_insert(:act)
      [indicator | _tail] = act.indicators

      activity = activity_fixture(@valid_attrs |> Map.put("act", act))

      all_attrs =
        @update_attrs
        |> Map.put("draft", true)
        |> Map.put("activity_indicators", %{
          "0" => %{
            "indicator_id" => indicator.id,
            "indicator_value" => "42"
          }
        })

      assert {:ok, %{activity: %Activity{} = activity}} = Monitoring.update_activity(activity, all_attrs)
      assert activity.draft == true

      # Check act last_activity_date is not updated
      act = Plan.get_act!(act.id, @prefix)
      assert act.last_activity_date != activity.date

      # Check indicator current value is not 40
      indicator = Monitoring.get_indicator!(indicator.id, @prefix)
      assert indicator.current_value != 42
    end

    test "update_activity/2 with valid data and published activty does not udate act not indicator value" do
      act = tenant_insert(:act)
      [indicator | _tail] = act.indicators

      activity = activity_fixture(@valid_attrs |> Map.put("act", act))

      all_attrs =
        @update_attrs
        |> Map.put("draft", false)
        |> Map.put("activity_indicators", %{
          "0" => %{
            "indicator_id" => indicator.id,
            "indicator_value" => "42"
          }
        })

      assert {:ok, %{activity: %Activity{} = activity}} = Monitoring.update_activity(activity, all_attrs)
      assert activity.draft == false

      # Check act last_activity_date is updated
      act = Plan.get_act!(act.id, @prefix)
      assert act.last_activity_date == activity.date

      # Check indicator current value is 42
      indicator = Monitoring.get_indicator!(indicator.id, @prefix)
      assert indicator.current_value == Decimal.new(42)
    end

    test "update_activity/2 with invalid data returns error changeset" do
      activity = activity_fixture() |> Repo.preload([activity_indicators: [indicator: :indicator_kind], act: :goal])
      assert {:error, :activity, %Ecto.Changeset{}, _} = Monitoring.update_activity(activity, @invalid_attrs)
      assert activity == Monitoring.get_activity!(activity.id, @prefix)
    end

    test "delete_activity/1 deletes the activity" do
      activity = activity_fixture()
      assert {:ok, _} = Monitoring.delete_activity(activity)

      assert_raise Ecto.NoResultsError, fn -> Monitoring.get_activity!(activity.id, @prefix) end
    end

    test "delete_activity/1 deletes the activity and updates act and indicator" do
      act = tenant_insert(:act)
      [indicator | _tail] = act.indicators

      activity = activity_fixture(@valid_attrs |> Map.put("act", act) |> Map.put("draft", false))

      act = Plan.get_act!(act.id, @prefix)
      assert act.last_activity_date == ~D[2010-04-17]
      date_before_delete = activity.date

      indicator = Monitoring.get_indicator!(indicator.id, @prefix)
      assert indicator.current_value == Decimal.new(40)

      assert {:ok, _} = Monitoring.delete_activity(activity)

      assert_raise Ecto.NoResultsError, fn -> Monitoring.get_activity!(activity.id, @prefix) end

      # Check act last activity date is updated
      act = Plan.get_act!(act.id, @prefix)
      assert act.last_activity_date != date_before_delete

      # Check indicator current value is updated
      indicator = Monitoring.get_indicator!(indicator.id, @prefix)
      assert indicator.current_value == nil
    end

    test "change_activity/1 returns a activity changeset" do
      activity = activity_fixture()
      assert %Ecto.Changeset{} = Monitoring.change_activity(activity)
    end
  end
end
