defmodule OgpEuskadi.PlanTest do
  use OgpEuskadi.DataCase

  alias OgpEuskadi.Repo

  alias OgpEuskadi.Basic
  alias OgpEuskadi.Plan
  alias OgpEuskadi.Plan.Act
  alias OgpEuskadi.Plan.Goal

  describe "goals" do
    @valid_goal_attrs %{ title_es: "Título objetivo (es)", description_es: "Descripción objetivo", draft: false}
    @update_goal_attrs %{title_es: "Nuevo título (es)", title_eu: "Título (eu)", description_es: "Descripción objetivo (es)", description_eu: "Descripción objetivo (eu)"}
    @invalid_goal_attrs %{title_es: nil, description_es: nil}

    def goal_fixture(attrs, prefix) do
      attrs =  attrs |> Enum.into(@valid_goal_attrs)

      {:ok, goal} = Plan.create_goal(attrs, prefix)

      goal
    end

    test "list_goals/1 returns all goals", %{prefix: prefix} do
      goal = goal_fixture(%{}, prefix)
      assert Plan.list_goals(prefix) == [goal]
    end

    test "get_goal!/2 returns the goal with given id", %{prefix: prefix} do
      goal = goal_fixture(%{}, prefix)
      assert Plan.get_goal!(goal.id, prefix) == goal
    end

    test "create_goal/2 with valid data creates a goal", %{prefix: prefix} do
      assert {:ok, %Goal{} = goal} = Plan.create_goal(@valid_goal_attrs, prefix)
    end

    test "create_goal/2 with invalid data returns error changeset", %{prefix: prefix} do
      assert {:error, %Ecto.Changeset{}} = Plan.create_goal(@invalid_goal_attrs, prefix)
    end

    test "update_goal/2 with valid data updates the goal", %{prefix: prefix} do
      goal = goal_fixture(%{}, prefix)
      assert {:ok, %Goal{} = goal} = Plan.update_goal(goal, @update_goal_attrs)
    end

    test "update_goal/2 with invalid data returns error changeset", %{prefix: prefix} do
      goal = goal_fixture(%{}, prefix)
      assert {:error, %Ecto.Changeset{}} = Plan.update_goal(goal, @invalid_goal_attrs)
      assert goal == Plan.get_goal!(goal.id, prefix)
    end

    test "delete_goal/1 deletes the goal", %{prefix: prefix} do
      goal = goal_fixture(%{}, prefix)
      assert {:ok, %Goal{}} = Plan.delete_goal(goal)
      assert_raise Ecto.NoResultsError, fn -> Plan.get_goal!(goal.id, prefix) end
    end

    test "change_goal/1 returns a goal changeset", %{prefix: prefix} do
      goal = goal_fixture(%{}, prefix)
      assert %Ecto.Changeset{} = Plan.change_goal(goal)
    end

  end

  describe "acts" do
    @valid_act_attrs %{ "title_es" => "Título Acción (es)", "description_es" => "Descripción de la acción (es)",  "starts_on_expected" => Date.add(Date.utc_today(), -60), "all_subdivisions" => true, "draft" => true}
    @update_act_attrs %{ "title_es" => "Nuevo título de  la acción (es)", "description_es" => "Nueva descripción de la acción (es)",  "starts_on" => Date.utc_today()}
    @invalid_act_attrs %{"title_es" => nil, "description_es" => nil, "starts_on" => nil}

    def act_fixture(attrs, prefix) do
      goal = attrs["goal"] || goal_fixture(%{}, prefix)
      topics_ids = Basic.list_topics(prefix) |> Enum.map(&(&1.id))
      attrs =  attrs  |> Map.put("goal_id", goal.id) |> Map.put("topics_ids", topics_ids) |> Enum.into(@valid_act_attrs)

      {:ok, act} = Plan.create_act(attrs, prefix)

      # Change fixture element values to match the DB values
      db_starts_on_expected = "#{String.pad_leading(Integer.to_string(act.starts_on_expected.day), 2, "0")}/#{String.pad_leading(Integer.to_string(act.starts_on_expected.month), 2, "0")}/#{act.starts_on_expected.year}"
      act =
        act
        |> Map.put(:starts_on_expected, db_starts_on_expected) # DB value has different format from the initial value
        |> Map.put(:subdivisions_ids, nil) # This attribute is set only on create/update
        |> Map.put(:ods_ids, nil)          # This attribute is set only on create/update
        |> Map.put(:topics_ids, nil)       # This attribute is set only on create/update
        |> Map.put(:discarded, false)      # By default discarded is false

      act
    end


    test "list_acts/1 returns all acts", %{prefix: prefix} do
      act = act_fixture(%{}, prefix) |> Repo.preload([:goal, :topics, :subdivisions, :indicators])
      assert Plan.list_acts(prefix) == [act]
    end

    test "get_act/2 returns the act with given id", %{prefix: prefix} do
      act = act_fixture(%{}, prefix) |> Plan.preload_act_associations()
      db_act = Plan.get_act!(act.id, prefix)

      assert db_act == act
    end

    test "create_act/2 with valid data creates an act", %{prefix: prefix} do
      goal = goal_fixture(%{}, prefix)
      topics_ids = Basic.list_topics(prefix) |> Enum.map(&(&1.id))
      attrs =  @valid_act_attrs  |> Map.put("goal_id", goal.id) |> Map.put("topics_ids", topics_ids)
      assert {:ok, %Act{} = act} = Plan.create_act(attrs, prefix)
    end

    test "create_act/2 with valid data assigns act last_activity_date", %{prefix: prefix} do
      goal = goal_fixture(%{}, prefix)
      topics_ids = Basic.list_topics(prefix) |> Enum.map(&(&1.id))
      attrs =  @valid_act_attrs  |> Map.put("goal_id", goal.id) |> Map.put("topics_ids", topics_ids)
      assert {:ok, %Act{} = act} = Plan.create_act(attrs, prefix)

      {:ok, inserted_at_date} = Date.new(act.inserted_at.year, act.inserted_at.month, act.inserted_at.day)
      assert act.last_activity_date == inserted_at_date
    end

    test "create_act/2 with invalid data returns error changeset", %{prefix: prefix} do
      assert {:error, %Ecto.Changeset{}} = Plan.create_act(@invalid_act_attrs, prefix)
    end

    test "update_act/2 with valid data updates the act", %{prefix: prefix} do
      act = act_fixture(%{}, prefix)
      attrs = @update_act_attrs |> Map.put("goal_id", act.goal_id) |> Map.put("topics_ids", act.topics |> Enum.map(&(&1.id)))
      assert {:ok, %Act{} = act} = Plan.update_act(act, attrs)
    end

    test "update_act/2 with valid data does not update the act last_activity_date", %{prefix: prefix} do
      act = act_fixture(%{"last_activity_date" => Date.add(Date.utc_today, -10)}, prefix)


      attrs = @update_act_attrs |> Map.put("topics_ids", act.topics |> Enum.map(&(&1.id))) |> Map.put("description_es", "New description")
      assert {:ok, %Act{} = updated_act} = Plan.update_act(act, attrs)

      assert updated_act.description_es == "New description"
      assert updated_act.last_activity_date == act.last_activity_date
      assert updated_act.updated_at != updated_act.last_activity_date
    end


    test "update_act/2 with invalid data returns error changeset", %{prefix: prefix} do
      act = act_fixture(%{}, prefix) |> Plan.preload_act_associations()
      assert {:error, %Ecto.Changeset{}} = Plan.update_act(act, @invalid_act_attrs)

      db_act = Plan.get_act!(act.id, prefix)
      assert act == db_act
    end

    test "delete_act/1 deletes the act", %{prefix: prefix} do
      act = act_fixture(%{}, prefix)
      assert {:ok, %Act{}} = Plan.delete_act(act)
      assert_raise Ecto.NoResultsError, fn -> Plan.get_act!(act.id, prefix) end
      ## #2DO - check that all acts_topics related to the deleted act are deleted
    end

    test "change_act/1 returns an act  changeset", %{prefix: prefix} do
      act = act_fixture(%{}, prefix)
      assert %Ecto.Changeset{} = Plan.change_act(act)
    end

  end
end
