# Based on https://thoughtbot.com/blog/building-a-phoenix-json-api
# Header setup based on https://www.mitchellhanberg.com/post/2017/11/28/implementing-api-authentication-with-guardian/
defmodule OgpEuskadiWeb.Api.ActControllerTest do
  use OgpEuskadiWeb.ConnCase

  alias OgpEuskadi.Auth
  alias OgpEuskadiWeb.Api.ActView
  alias OgpEuskadi.DateInput
  alias OgpEuskadi.TextDate

  describe "not authenticated client" do
    test "return unauthorized", %{conn: conn} do
      conn = get conn, Routes.api_act_path(conn, :index)

      assert json_response(conn, 401) == %{"error" => "unauthenticated"}
    end
  end

  describe "authenticated client" do
    setup %{conn: conn} do

      api_client = insert(:api_client_with_entity)

      # create the token
      {:ok, token, _claims} = Auth.authenticate_client_by_api_key!(api_client.client, api_client.key)

      # add authorization header to request
      conn = conn |> put_req_header("authorization", "Bearer #{token}")

      %{conn: conn}
    end

    test "#index renders a list of acts", %{conn: conn} do
      act = tenant_insert(:act, %{discarded: false, draft: false, all_subdivisions: false, topics: [], subdivisions: [], ods: []})

      conn = get conn, Routes.api_act_path(conn, :index)

      assert json_response(conn, 200) == render_json(ActView, "index.json", acts: [act])
    end

    test "#show renders a single act", %{conn: conn} do
      act = tenant_insert(:act_with_associations, %{discarded: false, draft: false, all_subdivisions: false})

      conn = get conn, Routes.api_act_path(conn, :show, act.id)

      # NOTE: render_json is not used to test that dates are converted to ISO8601 format.
      assert json_response(conn, 200) ==
        %{
          "act" => %{
            "goal" => %{
              "id" => act.goal.id,
              "title_es" => act.goal.title_es,
              "title_eu" => act.goal.title_eu
            },
            "id" => act.id,
            "title_es" => act.title_es,
            "title_eu" => act.title_eu,
            "description_es" => act.description_es,
            "description_eu" => act.description_eu,
            "starts_on" => DateInput.date_input_format(act.starts_on),
            "starts_on_expected" => DateInput.date_input_format(act.starts_on_expected),
            "ends_on" => DateInput.date_input_format(act.ends_on),
            "ends_on_expected" => DateInput.date_input_format(act.ends_on_expected),
            "draft" => act.draft,
            "discarded" => act.discarded,
            "position" => act.position,
            "topics" => Enum.map(act.topics, fn t -> %{"id" => t.id, "title_es" => t.title_es, "title_eu" => t.title_eu } end),
            "subdivisions" => Enum.map(act.subdivisions, fn s -> %{"id" => s.id, "name_es" => s.name_es, "name_eu" => s.name_eu} end),
            "ods" => Enum.map(act.ods, fn o -> %{"id" => o.id, "title_es" => o.title_es, "title_eu" => o.title_eu} end),
            "indicators" => Enum.map(act.indicators, fn act_ind ->
              ind = OgpEuskadi.JsonAPI.set_indicator_api_values(act_ind)
              %{
                "id" => ind.id,
                "title_es" => ind.title_es,
                "title_eu" => ind.title_eu,
                "type" => ind.indicator_kind.type,
                "goal_value" => ind.goal_value,
                "current_value_es" => ind.current_value_es,
                "current_value_eu" => ind.current_value_eu,
                "units_es" => ind.units_es,
                "units_eu" => ind.units_eu,
                "inserted_at" => NaiveDateTime.to_iso8601(ind.inserted_at),
                "updated_at" => NaiveDateTime.to_iso8601(ind.updated_at)
              }
            end),
            "inserted_at" => NaiveDateTime.to_iso8601(act.inserted_at),
            "updated_at" => NaiveDateTime.to_iso8601(act.updated_at)
          }
        }
    end

    test "#create creates an act and returns its data", %{conn: conn} do
      goal = tenant_insert(:goal)

      title_es = "Acción de prueba"
      description_es = "Una acción de prueba"
      starts_on_expected = ~D[2020-01-01] #"01/01/2020"
      topics = OgpEuskadi.Basic.list_topics("ogpe_test_tenant") |> Enum.take(2)

      conn = post conn, Routes.api_act_path(conn, :create, %{act: %{title_es: title_es, description_es: description_es, goal_id: goal.id, starts_on_expected: TextDate.get_text_date(starts_on_expected), topics_ids:  topics |> Enum.map(&(&1.id))}})

      assert %{"id" => id, "updated_at" => updated_at, "inserted_at" => inserted_at} = json_response(conn, 201)["act"]

      assert json_response(conn, 201) == %{
        "act" => %{
            "goal" => %{
              "id" => goal.id,
              "title_es" => goal.title_es,
              "title_eu" => goal.title_eu
            },
            "id" => id,
            "title_es" => title_es,
            "title_eu" => nil,
            "description_es" => description_es,
            "description_eu" => nil,
            "starts_on" => nil,
            "starts_on_expected" => DateInput.date_input_format(starts_on_expected),
            "ends_on" => nil,
            "ends_on_expected" => nil,
            "draft" => true,
            "discarded" => false,
            "position" => nil,
            "topics" => Enum.map(topics, fn t -> %{"id" => t.id, "title_es" => t.title_es, "title_eu" => t.title_eu } end),
            "subdivisions" => [],
            "ods" => [],
            "indicators" => [],
            "inserted_at" => inserted_at,
            "updated_at" => updated_at
        }
      }
    end

    test "#create returns validation errors with invalid data", %{conn: conn} do
      {status, headers, body} = assert_error_sent :unprocessable_entity, fn ->
        post conn, Routes.api_act_path(conn, :create, %{act: %{ title_es: "Acción de prueba"}})
      end

      assert body == Jason.encode!(
        %{
          "message" => "Data is not valid",
          "errors" => %{
            "description_es" => ["can't be blank"],
            "goal_id" => ["can't be blank"],
            "starts_on_expected" => ["can't be blank"],
            "topics_ids" => ["can't be blank"]
          }
        }
      )
    end

    test "#create returns missing parameters error when act parameter is missing", %{conn: conn} do
      {status, headers, body} = assert_error_sent :bad_request, fn ->
        post conn, Routes.api_act_path(conn, :create, %{})
      end

      assert body == Jason.encode!(
        %{
         "message" => "Missing parameters",
         "errors" => %{"act" => ["required parameter"]}
        }
      )
    end

    test "#update updates an act and returns updated data", %{conn: conn} do
      act = tenant_insert(:act_with_associations, %{discarded: false, draft: false, all_subdivisions: false})
      new_title_es = "Acción actualizada"
      new_description_es = "Descripción actualizada"

      conn = patch conn, Routes.api_act_path(conn, :update, act), %{act: %{ title_es: new_title_es, description_es: new_description_es}}

      assert %{"updated_at" => new_updated_at} = json_response(conn, 200)["act"]

      assert json_response(conn, 200) ==
        %{
          "act" => %{
            "goal" => %{
              "id" => act.goal.id,
              "title_es" => act.goal.title_es,
              "title_eu" => act.goal.title_eu
            },
            "id" => act.id,
            "title_es" => new_title_es,
            "title_eu" => act.title_eu,
            "description_es" => new_description_es,
            "description_eu" => act.description_eu,
            "starts_on" => DateInput.date_input_format(act.starts_on),
            "starts_on_expected" => DateInput.date_input_format(act.starts_on_expected),
            "ends_on" => DateInput.date_input_format(act.ends_on),
            "ends_on_expected" => DateInput.date_input_format(act.ends_on_expected),
            "draft" => act.draft,
            "discarded" => act.discarded,
            "position" => act.position,
            "topics" => Enum.map(act.topics, fn t -> %{"id" => t.id, "title_es" => t.title_es, "title_eu" => t.title_eu } end),
            "subdivisions" => Enum.map(act.subdivisions, fn s -> %{"id" => s.id, "name_es" => s.name_es, "name_eu" => s.name_eu} end),
            "ods" => Enum.map(act.ods, fn o -> %{"id" => o.id, "title_es" => o.title_es, "title_eu" => o.title_eu} end),
            "indicators" => Enum.map(act.indicators, fn act_ind ->
              ind = OgpEuskadi.JsonAPI.set_indicator_api_values(act_ind)
              %{
                "id" => ind.id,
                "title_es" => ind.title_es,
                "title_eu" => ind.title_eu,
                "type" => ind.indicator_kind.type,
                "goal_value" => ind.goal_value,
                "current_value_es" => ind.current_value_es,
                "current_value_eu" => ind.current_value_eu,
                "units_es" => ind.units_es,
                "units_eu" => ind.units_eu,
                "inserted_at" => NaiveDateTime.to_iso8601(ind.inserted_at),
                "updated_at" => NaiveDateTime.to_iso8601(ind.updated_at)
              }
            end),
            "inserted_at" => NaiveDateTime.to_iso8601(act.inserted_at),
            "updated_at" => new_updated_at
          }
        }
    end

    test "#update returns validation errors with invalid data", %{conn: conn} do
      act = tenant_insert(:act_with_associations, %{discarded: false, draft: false, all_subdivisions: false})

      {status, headers, body} = assert_error_sent :unprocessable_entity, fn ->
        patch conn, Routes.api_act_path(conn, :update, act), %{act: %{ title_es: ""}}
      end

      assert body == Jason.encode!(
        %{
          "message" => "Data is not valid",
          "errors" => %{"title_es" => ["can't be blank"]}
        }
      )
    end

    test "#update returns missing parameters if act parameter is not provided", %{conn: conn} do
      act = tenant_insert(:act_with_associations, %{discarded: false, draft: false, all_subdivisions: false})

      {status, headers, body} = assert_error_sent :bad_request, fn ->
        patch conn, Routes.api_act_path(conn, :update, act), %{}
      end

      assert body == Jason.encode!(
        %{
          "message" => "Missing parameters",
          "errors" => %{"act" => ["required parameter"]}
        }
      )
    end


    test "#delete deletes an act without activities", %{conn: conn} do
      act = tenant_insert(:act)

      conn = delete conn, Routes.api_act_path(conn, :delete, act)

      assert response(conn, 204)
    end

    test "#delete does not delete act with activities", %{conn: conn} do
      act = tenant_insert(:act)
      add_activity(act) |> tenant_insert()

      {status, headers, body} = assert_error_sent :forbidden, fn ->
        delete conn, Routes.api_act_path(conn, :delete, act)
      end

      assert body == Jason.encode!(
        %{
          "message" => "Action can not be executed",
          "errors": %{ "activities" => ["are still associated with this entry"] }
        }
      )
    end


    test "#delete returns not found if there is no act with the provided id", %{conn: conn} do
      {status, headers, body} = assert_error_sent 404, fn ->
        delete conn, Routes.api_act_path(conn, :delete, %OgpEuskadi.Plan.Act{id: 1000})
      end

      assert body == Jason.encode!(
        %{
          "message" => "Record not found. Please, check the ID.",
        }
      )
    end
  end
end
