defmodule OgpEuskadiWeb.Api.ActivityControllerTest do
  use OgpEuskadiWeb.ConnCase

  alias OgpEuskadi.Auth

  alias OgpEuskadiWeb.Api.ActivityView
  alias OgpEuskadi.DateInput
  alias OgpEuskadi.TextDate
  alias OgpEuskadi.Basic.IndicatorKind

  describe "not authenticated client" do
    test "return unauthorized", %{conn: conn} do
      act = tenant_insert(:act)
      conn = get conn, Routes.api_act_activity_path(conn, :index, act)

      assert json_response(conn, 401) == %{"error" => "unauthenticated"}
    end
  end

  describe "authenticated client" do
    setup %{conn: conn} do

      api_client = insert(:api_client_with_entity)

      # create the token
      {:ok, token, _claims} = Auth.authenticate_client_by_api_key!(api_client.client, api_client.key)

      # add authorization header to request
      conn = conn |> put_req_header("authorization", "Bearer #{token}")

      %{conn: conn}
    end

    test "#index renders a list of act activites", %{conn: conn} do
      act = tenant_insert(:act)
      activity = add_activity(act) |> tenant_insert()

      conn = get conn, Routes.api_act_activity_path(conn, :index, act)

      assert json_response(conn, 200) == render_json(ActivityView, "index.json", activities: [activity])
    end

    test "#index returns not found if there is no act with act_id", %{conn: conn} do

      {status, headers, body} = assert_error_sent 404, fn ->
        get conn, Routes.api_act_activity_path(conn, :index, %OgpEuskadi.Plan.Act{id: 1000})
      end

      assert body == Jason.encode!(
        %{
          "message" => "Record not found. Please, check the ID.",
        }
      )

    end

    test "#show renders a single acivity", %{conn: conn} do
      act = tenant_insert(:act)
      indicator = act.indicators |> Enum.at(0)
      activity = add_activity(act) |> tenant_insert()

      conn = get conn, Routes.api_activity_path(conn, :show, activity.id)

      assert json_response(conn, 200) == %{
        "activity" => %{
          "id" => activity.id,
          "act" => %{
            "id" => activity.act_id,
            "title_es" => act.title_es,
            "title_eu" => act.title_eu
          },
          "title_es" => activity.title_es,
          "title_eu" => activity.title_eu,
          "description_es" => activity.description_es,
          "description_eu" => activity.description_eu,
          "date" => DateInput.date_input_format(activity.date),
          "draft" => activity.draft,
          "activity_indicators" => [
            %{
              "indicator" => %{
                "id" => indicator.id,
                "title_es" => indicator.title_es,
                "title_eu" => indicator.title_eu,
                "units_es" => "%",
                "units_eu" => "%"
              },
              "indicator_value_es" => 1.0,
              "indicator_value_eu" => 1.0
            }
          ],
          "inserted_at" => NaiveDateTime.to_iso8601(activity.inserted_at),
          "updated_at" => NaiveDateTime.to_iso8601(activity.updated_at)
        }
      }
    end

    test "#create creates an activity and returns its data", %{conn: conn} do
      act = tenant_insert(:act_with_associations)
      indicator_prc = act.indicators |> Enum.find(&(&1.indicator_kind.type == "percentage"))
      indicator_num = act.indicators |> Enum.find(&(&1.indicator_kind.type == "numeric"))
      indicator_phs = act.indicators |> Enum.find(&(&1.indicator_kind.type == "phases"))


      activity_params = %{
        act_id: act.id,
        title_es: "Actividad de prueba",
        description_es: "Descripción de la actividad de prueba",
        draft: false,
        date: ~D[2020-09-10],
        activity_indicators: [
          %{
            indicator_id: indicator_prc.id,
            indicator_value: 1
          },
          %{
            indicator_id: indicator_num.id,
            indicator_value: 100
          },
          %{
            indicator_id: indicator_phs.id,
            indicator_value: 1
          }
        ]
      }

      activity_params =
        activity_params
        |> Map.replace(:date, TextDate.get_text_date(Map.get(activity_params, :date)))

      conn = post(conn |> put_req_header("content-type", "application/json"),
        Routes.api_activity_path(conn, :create),
        %{activity: activity_params} |> Jason.encode!()
      )

      assert %{"id" => id, "updated_at" => updated_at, "inserted_at" => inserted_at} = json_response(conn, 201)["activity"]

      assert json_response(conn, 201) == %{
        "activity" => %{
          "id" => id,
          "act" => %{
            "id" => act.id,
            "title_es" => act.title_es,
            "title_eu" => act.title_eu
          },
          "title_es" => Map.get(activity_params, :title_es),
          "title_eu" => nil,
          "description_es" => Map.get(activity_params, :description_es),
          "description_eu" => nil,
          "date" => DateInput.date_input_format(Map.get(activity_params, :date)),
          "activity_indicators" => [
            %{
              "indicator" => %{
                "id" => indicator_prc.id,
                "title_es" => indicator_prc.title_es,
                "title_eu" => indicator_prc.title_eu,
                "units_es" => "%",
                "units_eu" => "%"
              },
              "indicator_value_es" => 1.0,
              "indicator_value_eu" => 1.0
            },
            %{
              "indicator" => %{
                "id" => indicator_num.id,
                "title_es" => indicator_num.title_es,
                "title_eu" => indicator_num.title_eu,
                "units_es" => indicator_num.units_es,
                "units_eu" => indicator_num.units_eu
              },
              "indicator_value_es" => 100.0,
              "indicator_value_eu" => 100.0
            },
            %{
              "indicator" => %{
                "id" => indicator_phs.id,
                "title_es" => indicator_phs.title_es,
                "title_eu" => indicator_phs.title_eu,
                "units_es" => nil,
                "units_eu" => nil
              },
              "indicator_value_es" => Enum.at(IndicatorKind.options_list(indicator_phs.indicator_kind, "es"), 1),
              "indicator_value_eu" => Enum.at(IndicatorKind.options_list(indicator_phs.indicator_kind, "eu"), 1)
            }          ],
          "draft" => Map.get(activity_params, :draft),
          "inserted_at" => inserted_at,
          "updated_at" => updated_at
        }
      }
    end

    test "#create returns validation errors with invalid data", %{conn: conn} do

      act = tenant_insert(:act)

      {status, headers, body} = assert_error_sent :unprocessable_entity, fn ->
        post conn, Routes.api_activity_path(conn, :create, %{act_id: act.id, activity: %{ title_es: "Actividad incompleta"}})
      end

      assert body == Jason.encode!(
        %{
          "message" => "Data is not valid",
          "errors" => %{
            "act_id" => ["can't be blank"],
            "activity_indicators" => ["can't be blank"],
            "date" => ["can't be blank"],
            "description_es" => ["can't be blank"],
            "activity_indicators" => ["can't be blank"],
          }
        }
      )
    end

    test "#create returns missing parameters error when activity parameters are missing", %{conn: conn} do
      act = tenant_insert(:act)
      {status, headers, body} = assert_error_sent :bad_request, fn ->
        post conn, Routes.api_activity_path(conn, :create, %{})
      end

      assert body == Jason.encode!(
        %{
         "message" => "Missing parameters",
         "errors" => %{"activity" => ["required parameter"]}
        }
      )
    end

    test "#update updates an activity and returns updated data", %{conn: conn} do
      act = tenant_insert(:act)
      indicator = act.indicators |> Enum.at(0)
      activity = add_activity(act) |> tenant_insert()

      activity_params = %{
        act_id: act.id,
        date: activity.date,
        title_es: "Actividad actualizada",
        description_es: "Descripción de la actividad actualizada",
        draft: !activity.draft,
        activity_indicators: %{
          "0" => %{
            indicator_id: indicator.id,
            indicator_value: 10
          }
        }
      }

      conn = patch conn, Routes.api_activity_path(conn, :update, activity), %{activity: activity_params}

      assert %{"updated_at" => new_updated_at} = json_response(conn, 200)["activity"]

      assert json_response(conn, 200) == %{
        "activity" => %{
          "id" => activity.id,
          "act" => %{
            "id" => act.id,
            "title_es" => act.title_es,
            "title_eu" => act.title_eu
          },
          "title_es" => Map.get(activity_params, :title_es),
          "title_eu" => nil,
          "description_es" => Map.get(activity_params, :description_es),
          "description_eu" => nil,
          "date" => DateInput.date_input_format(Map.get(activity_params, :date)),
          "draft" => Map.get(activity_params, :draft),
          "activity_indicators" => [
            %{
              "indicator" => %{
                "id" => indicator.id,
                "title_es" => indicator.title_es,
                "title_eu" => indicator.title_eu,
                "units_es" => "%",
                "units_eu" => "%"
              },
              "indicator_value_es" => 10.0,
              "indicator_value_eu" => 10.0
            }
          ],
          "inserted_at" => NaiveDateTime.to_iso8601(activity.inserted_at),
          "updated_at" => new_updated_at
        }
      }
    end

    test "#update updates only the values passed as parameters and returns updated data", %{conn: conn} do
      act = tenant_insert(:act)
      indicator = act.indicators |> Enum.at(0)
      activity = add_activity(act) |> tenant_insert()

      activity_params = %{
        title_es: "Actividad actualizada",
        description_es: "Descripción de la actividad actualizada"
      }

      conn = patch conn, Routes.api_activity_path(conn, :update, activity), %{activity: activity_params}

      assert %{"updated_at" => new_updated_at} = json_response(conn, 200)["activity"]

      assert json_response(conn, 200) == %{
        "activity" => %{
          "id" => activity.id,
          "act" => %{
            "id" => act.id,
            "title_es" => act.title_es,
            "title_eu" => act.title_eu
          },
          "title_es" => Map.get(activity_params, :title_es),
          "title_eu" => nil,
          "description_es" => Map.get(activity_params, :description_es),
          "description_eu" => nil,
          "date" => DateInput.date_input_format(activity.date),
          "draft" => activity.draft,
          "activity_indicators" => activity.activity_indicators |> Enum.map(fn ai ->
            %{
              "indicator" => %{
                "id" => ai.indicator_id,
                "title_es" => ai.indicator.title_es,
                "title_eu" => ai.indicator.title_eu,
                "units_es" => "%",
                "units_eu" => "%"
              },
              "indicator_value_es" => 1.0,
              "indicator_value_eu" => 1.0
             }
          end),
          "inserted_at" => NaiveDateTime.to_iso8601(activity.inserted_at),
          "updated_at" => new_updated_at
        }
      }
    end

    test "#update returns validation errors with invalid data", %{conn: conn} do
      act = tenant_insert(:act)
      activity = add_activity(act) |> tenant_insert()

      activity_params = %{
        title_es: ""
      }


      {status, headers, body} = assert_error_sent :unprocessable_entity, fn ->
        patch conn, Routes.api_activity_path(conn, :update, activity), %{activity: activity_params}
      end

      assert body == Jason.encode!(
        %{
          "message" => "Data is not valid",
          "errors" => %{"title_es" => ["can't be blank"]}
        }
      )
    end

    test "#update returns missing parameters if activity parameters is not provided", %{conn: conn} do
      act = tenant_insert(:act)
      activity = add_activity(act) |> tenant_insert()

      {status, headers, body} = assert_error_sent :bad_request, fn ->
        patch conn, Routes.api_activity_path(conn, :update, activity), %{}
      end

      assert body == Jason.encode!(
        %{
          "message" => "Missing parameters",
          "errors" => %{
            "activity" => ["required parameter"]
          }
        }
      )
    end

    test "#delete deletes an activity", %{conn: conn} do
      act = tenant_insert(:act)
      activity = add_activity(act) |> tenant_insert()

      conn = delete conn, Routes.api_activity_path(conn, :delete, activity)

      assert response(conn, 204)
    end
  end

end
