# Based on https://thoughtbot.com/blog/building-a-phoenix-json-api
# Header setup based on https://www.mitchellhanberg.com/post/2017/11/28/implementing-api-authentication-with-guardian/
defmodule OgpEuskadiWeb.Api.GoalControllerTest do
  use OgpEuskadiWeb.ConnCase

  alias OgpEuskadi.Auth
  alias OgpEuskadi.Plan
  alias OgpEuskadiWeb.Api.GoalView

  @prefix "ogpe_test_tenant"

  describe "not authenticaed client" do
    test "return unauthorized", %{conn: conn} do
      conn = get conn, Routes.api_goal_path(conn, :index)

      assert json_response(conn, 401) == %{"error" => "unauthenticated"}
    end
  end

  describe "authenticated client" do
    setup %{conn: conn} do

      api_client = insert(:api_client_with_entity)

      # create the token
      {:ok, token, _claims} = Auth.authenticate_client_by_api_key!(api_client.client, api_client.key)

      # add authorization header to request
      conn = conn |> put_req_header("authorization", "Bearer #{token}")

      %{conn: conn}
    end

    test "#index renders a list of goals", %{conn: conn} do
      goal = tenant_insert(:goal)

      conn = get conn, Routes.api_goal_path(conn, :index)

      assert json_response(conn, 200) == render_json(GoalView, "index.json", goals: [goal])
    end

    test "#show renders a single goal", %{conn: conn} do
      goal = tenant_insert(:goal)

      conn = get conn, Routes.api_goal_path(conn, :show, goal.id)

      # NOTE: render_json is not used to test that dates are converted to ISO8601 format.
      assert json_response(conn, 200) ==
        %{
          "goal" => %{
            "id" => goal.id,
            "title_es" => goal.title_es,
            "title_eu" => goal.title_eu,
            "description_es" => goal.description_es,
            "description_eu" => goal.description_eu,
            "position" => goal.position,
            "draft" => goal.draft,
            "inserted_at" => NaiveDateTime.to_iso8601(goal.inserted_at),
            "updated_at" => NaiveDateTime.to_iso8601(goal.updated_at)
          }
        }
    end

    test "#show returns error if invalid id is provided", %{conn: conn} do
      {status, headers, body} = assert_error_sent 400, fn ->
        get conn, Routes.api_goal_path(conn, :show, "xx")
      end

      assert body == Jason.encode!(
        %{
          "message" => "Wrong parameters",
          "errors" => [
             %{
               "name" => "id",
               "value" => "xx",
               "message" => "Value is not correct"
             }
          ]
        }
      )
    end

    test "#show returns error if record not found", %{conn: conn} do
      {status, headers, body} = assert_error_sent 404, fn ->
        get conn, Routes.api_goal_path(conn, :show, "100")
      end

      assert body == Jason.encode!(
        %{
          "message" => "Record not found. Please, check the ID.",
        }
      )
    end

    test "#create creates a goal and returns its data", %{conn: conn} do
      conn = post conn, Routes.api_goal_path(conn, :create, %{goal: %{ title_es: "Objetivo de prueba", description_es: "Un objetivo de prueba para el API"}})
      goal = Plan.list_goals(@prefix) |> Enum.fetch!(-1)

      assert json_response(conn, 201) == %{
        "goal" => %{
          "id" => goal.id,
          "title_es" => "Objetivo de prueba",
          "title_eu" => nil,
          "description_es" => "Un objetivo de prueba para el API",
          "description_eu" => nil,
          "draft" => nil,
          "position" => nil,
          "inserted_at" => NaiveDateTime.to_iso8601(goal.inserted_at),
          "updated_at" => NaiveDateTime.to_iso8601(goal.updated_at)
        }
      }
    end

    test "#create returns validation errors with invalid data", %{conn: conn} do
      {status, headers, body} = assert_error_sent :unprocessable_entity, fn ->
        post conn, Routes.api_goal_path(conn, :create, %{goal: %{ title_es: "Objetivo de prueba"}})
      end

      assert body == Jason.encode!(
        %{
          "message" => "Data is not valid",
          "errors" => %{"description_es" => ["can't be blank"]}
        }
      )
    end

    test "#create returns missing parameters error when goal parameter is missing", %{conn: conn} do
      {status, headers, body} = assert_error_sent :bad_request, fn ->
        post conn, Routes.api_goal_path(conn, :create, %{})
      end

      assert body == Jason.encode!(
        %{
          "message" => "Missing parameters",
          "errors" => %{"goal" => ["required parameter"]}
        }
      )
    end

    test "#update updates a goal and returns updated data", %{conn: conn} do
      goal = tenant_insert(:goal)
      conn = patch conn, Routes.api_goal_path(conn, :update, goal), %{goal: %{ title_es: "Objetivo actualizado", description_es: "Descripción actualizada"}}

      updated_goal = Plan.get_goal!(goal.id, @prefix)

      assert json_response(conn, 200) == %{
        "goal" => %{
          "id" => goal.id,
          "title_es" => "Objetivo actualizado",
          "title_eu" => goal.title_eu,
          "description_es" => "Descripción actualizada",
          "description_eu" => goal.description_eu,
          "draft" => goal.draft,
          "position" => goal.position,
          "inserted_at" => NaiveDateTime.to_iso8601(goal.inserted_at),
          "updated_at" => NaiveDateTime.to_iso8601(updated_goal.updated_at)
        }
      }
    end

    test "#update returns validation errors with invalid data", %{conn: conn} do
      goal = tenant_insert(:goal)

      {status, headers, body} = assert_error_sent :unprocessable_entity, fn ->
        patch conn, Routes.api_goal_path(conn, :update, goal), %{goal: %{ title_es: ""}}
      end

      assert body == Jason.encode!(
        %{
          "message" => "Data is not valid",
          "errors" => %{"title_es" => ["can't be blank"]}
        }
      )
    end

    test "#update returns missing parameters if goal parameter is not provided", %{conn: conn} do
      goal = tenant_insert(:goal)

      {status, headers, body} = assert_error_sent :bad_request, fn ->
        patch conn, Routes.api_goal_path(conn, :update, goal), %{}
      end

      assert body == Jason.encode!(
        %{
          "message" => "Missing parameters",
          "errors" => %{"goal" => ["required parameter"]}
        }
      )
    end

    test "#delete deletes a goal without actions", %{conn: conn} do
      goal = tenant_insert(:goal)

      conn = delete conn, Routes.api_goal_path(conn, :delete, goal)

      assert response(conn, 204)
    end

    test "#delete returns forbidden for a goal with actions", %{conn: conn} do
      goal = tenant_insert(:goal)
      act = tenant_insert(:act, goal: goal)

      {status, headers, body} = assert_error_sent :forbidden, fn ->
        delete conn, Routes.api_goal_path(conn, :delete, goal)
      end

      assert body == Jason.encode!(
        %{
          "message" => "Action can not be executed",
          "errors": %{ "acts" => ["are still associated with this entry"] }
        }
      )
    end

    test "#delete returns not found if there is no goal with the provided id", %{conn: conn} do
      {status, headers, body} = assert_error_sent 404, fn ->
        delete conn, Routes.api_goal_path(conn, :delete, %OgpEuskadi.Plan.Goal{id: 1000})
      end

      assert body == Jason.encode!(
        %{
          "message" => "Record not found. Please, check the ID.",
        }
      )
    end

  end
end
