defmodule OgpEuskadiWeb.Api.IndicatorControllerTest do
  use OgpEuskadiWeb.ConnCase

  alias OgpEuskadi.Auth

  alias OgpEuskadiWeb.Api.IndicatorView
  alias OgpEuskadi.Basic
  alias OgpEuskadi.Basic.IndicatorKind
  alias OgpEuskadi.Monitoring

  @prefix "ogpe_test_tenant"

  describe "not authenticated client" do
    test "return unauthorized", %{conn: conn} do
      conn = get conn, Routes.api_act_path(conn, :index)

      assert json_response(conn, 401) == %{"error" => "unauthenticated"}
    end
  end

  describe "authenticated client" do
    setup %{conn: conn} do

      api_client = insert(:api_client_with_entity)

      # create the token
      {:ok, token, _claims} = Auth.authenticate_client_by_api_key!(api_client.client, api_client.key)

      # add authorization header to request
      conn = conn |> put_req_header("authorization", "Bearer #{token}")

      %{conn: conn}
    end

    test "#index returns a list of act indicators", %{conn: conn} do
      act = tenant_insert(:act)

      conn = get conn, Routes.api_act_indicator_path(conn, :index, act)

      assert json_response(conn, 200) == render_json(IndicatorView, "index.json", indicators: act.indicators |> Enum.map(fn ind -> Monitoring.get_indicator!(ind.id, @prefix) end))
    end

    test "#show renders a single indicator", %{conn: conn} do
      act = tenant_insert(:act)
      indicator = act.indicators |> Enum.at(0)

      conn = get conn, Routes.api_indicator_path(conn, :show, indicator.id)

      assert json_response(conn, 200)["indicator"] == %{
        "id" => indicator.id,
        "act" => %{
          "id" => act.id,
          "title_es" => act.title_es,
          "title_eu" => act.title_eu
        },
        "indicator_kind" => %{
          "id" => indicator.indicator_kind.id,
          "type" => indicator.indicator_kind.type
        },
        "title_es" => indicator.title_es,
        "title_eu" => indicator.title_eu,
        "current_value_es" => 0.0,
        "current_value_eu" => 0.0,
        "goal_value" => nil,
        "units_es" => "%",
        "units_eu" => "%",
         "inserted_at" => NaiveDateTime.to_iso8601(indicator.inserted_at),
         "updated_at" => NaiveDateTime.to_iso8601(indicator.updated_at)
      }
    end

    test "#creates a porcentage type indicator for the given act and returns its data", %{conn: conn} do
      act = tenant_insert(:act)
      kind = Basic.list_indicator_kinds(@prefix) |> Enum.find(&(&1.type == "percentage"))

      indicator_params = %{
        act_id: act.id,
        title_es: "Porcentaje cumplimiento",
        kind_id: kind.id
      }

      conn = post conn, Routes.api_indicator_path(conn, :create, %{indicator: indicator_params})

      assert %{"id" => new_indicator_id, "inserted_at" => new_inserted_at, "updated_at" => new_updated_at} = json_response(conn, 201)["indicator"]

      assert json_response(conn, 201)["indicator"] == %{
        "id" => new_indicator_id,
        "act" => %{
          "id" => act.id,
          "title_es" => act.title_es,
          "title_eu" => act.title_eu
        },
        "indicator_kind" => %{
          "id" => kind.id,
          "type" => kind.type
        },
        "title_es" => "Porcentaje cumplimiento",
        "title_eu" => nil,
        "current_value_es" => 0.0,
        "current_value_eu" => 0.0,
        "goal_value" => nil,
        "units_es" => "%",
        "units_eu" => "%",
        "inserted_at" => new_inserted_at,
        "updated_at" => new_updated_at
      }
    end

    test "#creates a numeric type indicator for the given act and returns act data with indicators lists", %{conn: conn} do
      act = tenant_insert(:act)
      kind = Basic.list_indicator_kinds(@prefix) |> Enum.find(&(&1.type == "numeric"))

      indicator_params = %{
        act_id: act.id,
        title_es: "Contador",
        kind_id: kind.id,
        goal_value: 1000,
        units_es: "personas",
        units_eu: "pertzona"
      }

      conn = post conn, Routes.api_indicator_path(conn, :create, %{indicator: indicator_params})

      assert %{"id" => new_indicator_id, "inserted_at" => new_inserted_at, "updated_at" => new_updated_at} = json_response(conn, 201)["indicator"]

      assert json_response(conn, 201)["indicator"] == %{
        "id" => new_indicator_id,
        "act" => %{
          "id" => act.id,
          "title_es" => act.title_es,
          "title_eu" => act.title_eu
        },
        "indicator_kind" => %{
          "id" => kind.id,
          "type" => kind.type
        },
        "title_es" => "Contador",
        "title_eu" => nil,
        "current_value_es" => 0.0,
        "current_value_eu" => 0.0,
        "goal_value" => 1000.0,
        "units_es" => "personas",
        "units_eu" => "pertzona",
        "inserted_at" => new_inserted_at,
        "updated_at" => new_updated_at
      }
    end

    test "#creates a phases type indicator for the given act and returns act data with indicators lists", %{conn: conn} do
      act = tenant_insert(:act)
      kind = Basic.list_indicator_kinds(@prefix) |> Enum.find(&(&1.type == "phases"))

      indicator_params = %{
        act_id: act.id,
        title_es: "Indicador por fases de prueba",
        kind_id: kind.id
      }

      conn = post conn, Routes.api_indicator_path(conn, :create, %{indicator: indicator_params})

      assert %{"id" => new_indicator_id, "inserted_at" => new_inserted_at, "updated_at" => new_updated_at} = json_response(conn, 201)["indicator"]

      assert json_response(conn, 201)["indicator"] == %{
        "id" => new_indicator_id,
        "act" => %{
          "id" => act.id,
          "title_es" => act.title_es,
          "title_eu" => act.title_eu
        },
        "indicator_kind" => %{
          "id" => kind.id,
          "type" => kind.type
        },
        "title_es" => "Indicador por fases de prueba",
        "title_eu" => nil,
        "current_value_es" => IndicatorKind.options_list(kind, "es")|> Enum.at(0),
        "current_value_eu" => IndicatorKind.options_list(kind, "eu")|> Enum.at(0),
        "goal_value" => nil,
        "units_es" => nil,
        "units_eu" => nil,
        "inserted_at" => new_inserted_at,
        "updated_at" => new_updated_at
      }
    end

    test "#create returns validation errors with blank data", %{conn: conn} do
      {status, headers, body} = assert_error_sent :unprocessable_entity, fn ->
        post conn, Routes.api_indicator_path(conn, :create, %{indicator: %{ title_es: ""}})
      end

      assert body == Jason.encode!(
        %{
          "message" => "Data is not valid",
          "errors" => %{
            "title_es" => ["can't be blank"],
            "act_id" => ["can't be blank"],
            "kind_id" => ["can't be blank"]
          }
        }
      )
    end

    test "#create returns validation errors with invalid act_id", %{conn: conn} do
      {status, headers, body} = assert_error_sent :unprocessable_entity, fn ->
        post conn, Routes.api_indicator_path(conn, :create, %{indicator: %{ title_es: "Datos no válidos", act_id: 1000, kind_id: 1}})
      end

      assert body == Jason.encode!(
        %{
          "message" => "Data is not valid",
          "errors" => %{
            "act_id" => ["does not exist"],
          }
        }
      )
    end

    test "#create returns validation errors with invalid kind_id", %{conn: conn} do
      act = tenant_insert(:act)
      {status, headers, body} = assert_error_sent :unprocessable_entity, fn ->
        post conn, Routes.api_indicator_path(conn, :create, %{indicator: %{ title_es: "Datos no válidos", act_id: act.id, kind_id: 1000}})
      end

      assert body == Jason.encode!(
        %{
          "message" => "Data is not valid",
          "errors" => %{
            "kind_id" => ["does not exist"],
          }
        }
      )
    end

    test "#create returns missing parameters error when indicator parameter is missing", %{conn: conn} do
      {status, headers, body} = assert_error_sent :bad_request, fn ->
        post conn, Routes.api_indicator_path(conn, :create, %{})
      end

      assert body == Jason.encode!(
        %{
         "message" => "Missing parameters",
         "errors" => %{"indicator" => ["required parameter"]}
        }
      )
    end

    test "#update updates indicator title and returns indicator data", %{conn: conn} do
      act = tenant_insert(:act)
      indicator = act.indicators |> Enum.at(0)

      indicator_params = %{
        title_es: "#{indicator.title_es} (modificado)",
      }

      conn = patch conn, Routes.api_indicator_path(conn, :update, indicator), %{indicator: indicator_params}


      assert %{"id" => id, "updated_at" => new_updated_at} = json_response(conn, 200)["indicator"]

      initial_indicator = OgpEuskadi.JsonAPI.set_indicator_api_values(indicator)
      assert json_response(conn, 200)["indicator"] == %{
        "id" => indicator.id,
        "act" => %{
          "id" => act.id,
          "title_es" => act.title_es,
          "title_eu" => act.title_eu
        },
        "indicator_kind" => %{
          "id" => indicator.kind_id,
          "type" => indicator.indicator_kind.type
        },
        "title_es" => Map.get(indicator_params, :title_es),
        "title_eu" => initial_indicator.title_eu,
        "current_value_es" => initial_indicator.current_value_es,
        "current_value_eu" => initial_indicator.current_value_eu,
        "goal_value" => initial_indicator.goal_value,
        "units_es" => initial_indicator.units_es,
        "units_eu" => initial_indicator.units_eu,
        "inserted_at" => NaiveDateTime.to_iso8601(initial_indicator.inserted_at),
        "updated_at" => new_updated_at
      }
    end

    test "#update updates indicator type and units and returns indicator data", %{conn: conn} do
      act = tenant_insert(:act)
      indicator = act.indicators |> Enum.at(0)
      kind = Basic.list_indicator_kinds(@prefix) |> Enum.find(&(&1.type == "numeric"))

      indicator_params = %{
        title_es: "Contador",
        title_eu: "Contador (eu)",
        kind_id: kind.id,
        goal_value: 1000.0,
        units_es: "personas",
        units_eu: "pertzona"
      }

      conn = patch conn, Routes.api_indicator_path(conn, :update, indicator), %{indicator: indicator_params}

      updated_indicator = json_response(conn, 200)["indicator"]

      initial_indicator = OgpEuskadi.JsonAPI.set_indicator_api_values(indicator)
      assert updated_indicator == %{
        "id" => initial_indicator.id,
        "act" => %{
          "id" => act.id,
          "title_es" => act.title_es,
          "title_eu" => act.title_eu
        },
        "indicator_kind" => %{
          "id" => kind.id,
          "type" => kind.type
        },
        "title_es" => Map.get(indicator_params, :title_es),
        "title_eu" => Map.get(indicator_params, :title_eu),
        "current_value_es" => initial_indicator.current_value_es,
        "current_value_eu" => initial_indicator.current_value_eu,
        "goal_value" => Map.get(indicator_params, :goal_value),
        "units_es" => Map.get(indicator_params, :units_es),
        "units_eu" => Map.get(indicator_params, :units_eu),
        "inserted_at" => NaiveDateTime.to_iso8601(initial_indicator.inserted_at),
        "updated_at" => updated_indicator["updated_at"]
      }
    end

    test "#update returns validation errors with invalid data", %{conn: conn} do
      act = tenant_insert(:act)
      indicator = act.indicators |> Enum.at(0)

      indicator_params = %{
        title_es: "",
      }


      {status, headers, body} = assert_error_sent :unprocessable_entity, fn ->
        patch conn, Routes.api_indicator_path(conn, :update, indicator), %{indicator: indicator_params}
      end

      assert body == Jason.encode!(
        %{
          "message" => "Data is not valid",
          "errors" => %{"title_es" => ["can't be blank"]}
        }
      )
    end

    test "#update returns missing parameters if indicator parameter is not provided", %{conn: conn} do
      act = tenant_insert(:act)
      indicator = act.indicators |> Enum.at(0)

      {status, headers, body} = assert_error_sent :bad_request, fn ->
        patch conn, Routes.api_indicator_path(conn, :update, indicator), %{}
      end

      assert body == Jason.encode!(
        %{
          "message" => "Missing parameters",
          "errors" => %{"indicator" => ["required parameter"]}
        }
      )
    end

    test "#update type returns validation errors for indicator with activities", %{conn: conn} do
      act = tenant_insert(:act)
      indicator = act.indicators |> Enum.at(0)
      activity = add_activity(act) |> tenant_insert()

      kind = Basic.list_indicator_kinds(@prefix) |> Enum.find(&(&1.type == "numeric"))

      assert indicator.kind_id != kind.id

      indicator_params = %{
        kind_id: kind.id
      }


      {status, headers, body} = assert_error_sent :unprocessable_entity, fn ->
        patch conn, Routes.api_indicator_path(conn, :update, indicator), %{indicator: indicator_params}
      end

      assert body == Jason.encode!(
        %{
          "message" => "Data is not valid",
          "errors" => %{"kind_id" => ["can not be changed for indicator with activities"]}
        }
      )
    end

    test "#update does not update current_value", %{conn: conn} do
      act = tenant_insert(:act)
      indicator = act.indicators |> Enum.at(0)

      indicator_params = %{
        current_value: 10
      }


      {status, headers, body} = assert_error_sent :unprocessable_entity, fn ->
        patch conn, Routes.api_indicator_path(conn, :update, indicator), %{indicator: indicator_params}
      end

      assert body == Jason.encode!(
        %{
          "message" => "Data is not valid",
          "errors" => %{"current_value" => ["can not be updated directly. Please, use activities to update indicator values"]}
        }
      )
    end

    test "#delete deletes an indicator without activities", %{conn: conn} do
      act = tenant_insert(:act)

      conn = delete conn, Routes.api_indicator_path(conn, :delete, act.indicators |> Enum.at(0))

      assert response(conn, 204)

      assert length(OgpEuskadi.Plan.get_act!(act.id, @prefix).indicators) == 0
    end

    test "#delete does not delete indicator with activities", %{conn: conn} do
      act = tenant_insert(:act)
      indicator = act.indicators |> Enum.at(0)
      activity = add_activity(act) |> tenant_insert()

      {status, headers, body} = assert_error_sent :forbidden, fn ->
        delete conn, Routes.api_indicator_path(conn, :delete, indicator)
      end

      assert body == Jason.encode!(
        %{
          "message" => "Action can not be executed",
          "errors": %{ "activity_indicators" => ["are still associated with this entry"] }
        }
      )
    end

    test "#delete returns not found if there is no indicator with the provided id", %{conn: conn} do
      {status, headers, body} = assert_error_sent 404, fn ->
        delete conn, Routes.api_indicator_path(conn, :delete, %Monitoring.Indicator{id: 1000})
      end

      assert body == Jason.encode!(
        %{
          "message" => "Record not found. Please, check the ID.",
        }
      )
    end
  end
end
