# ExMacina Factory definitions
# Documentation: https://hexdocs.pm/ex_machina/readme.html
defmodule OgpEuskadi.Factory do
  use ExMachina.Ecto, repo: OgpEuskadi.Repo
  use OgpEuskadi.TenantStrategy

  import Ecto.Query

  def credential_factory do
    %OgpEuskadi.Accounts.Credential{
      email: sequence(:email, &"test-#{&1}@ogpeuskadi.org"),
      password: "secret",
      password_confirmation: "secret"
    }
  end

  def user_factory do
    %OgpEuskadi.Accounts.User{
      username: sequence(:username, &"Test #{&1}"),
      credential: build(:credential)
    }
  end

  def entity_factory do
    %OgpEuskadi.Site.Entity{
      tenant: "test_tenant",
      name_es: "Ayntamiento de Obaba",
      name_eu: "Obabako Udaletxea",
      published: true
    }
  end

  def goal_factory do
    title_es = sequence(:title_es, &"Objetivo #{&1}")
    %OgpEuskadi.Plan.Goal{
      title_es: title_es,
      description_es: "Descripción del objetivo #{title_es}",
      draft: false
    }
  end

  def act_factory do
    title_es = sequence(:title_es, &"Acción #{&1}")
    %OgpEuskadi.Plan.Act{
      title_es: title_es,
      description_es: "Descripción de la acción #{title_es}",
      starts_on_expected: Date.utc_today,
      goal: build(:goal),
      indicators: [build(:indicator)]
    }
  end

  # derived factory
  # def act_with_activity_factory do
  #   struct!(
  #     act_factory(),
  #     %{
  #       activities: [build(:activity)]
  #     }
  #   )
  # end

  def act_with_associations_factory do
    struct!(
      act_factory(),
      %{
        topics: OgpEuskadi.Basic.list_topics("ogpe_test_tenant") |> Enum.take(2),
        subdivisions: [build(:subdivision)],
        ods: OgpEuskadi.Basic.list_ods("ogpe_test_tenant") |> Enum.take(2),
        indicators: [build(:indicator_prc), build(:indicator_numeric), build(:indicator_phases)]
      }
    )
  end

  def indicator_factory do
    title_es = sequence(:title_es, &"Indicador #{&1}")
    indicator_kind = OgpEuskadi.Basic.IndicatorKind |> OgpEuskadi.Repo.get_by([type: "percentage"], prefix: "ogpe_test_tenant")

    %OgpEuskadi.Monitoring.Indicator{
      title_es: title_es,
      indicator_kind: indicator_kind,
      current_value: Decimal.new(0)
    }
  end

  def indicator_prc_factory do
    title_es = sequence(:title_es, &"Indicador #{&1}")
    indicator_kind = OgpEuskadi.Basic.IndicatorKind |> OgpEuskadi.Repo.get_by([type: "percentage"], prefix: "ogpe_test_tenant")

    %OgpEuskadi.Monitoring.Indicator{
      title_es: title_es,
      indicator_kind: indicator_kind,
      current_value: Decimal.new(25)
    }
  end

  def indicator_numeric_factory do
    title_es = sequence(:title_es, &"Indicador #{&1}")
    indicator_kind = OgpEuskadi.Basic.IndicatorKind |> OgpEuskadi.Repo.get_by([type: "numeric"], prefix: "ogpe_test_tenant")

    %OgpEuskadi.Monitoring.Indicator{
      title_es: title_es,
      indicator_kind: indicator_kind,
      current_value: Decimal.new(20),
      units_es: "personas",
      units_eu: "pertzona"
    }
  end

  def indicator_phases_factory do
    title_es = sequence(:title_es, &"Indicador #{&1}")
    indicator_kind =
      from(ik in OgpEuskadi.Basic.IndicatorKind, where: [type: "phases"])
      |> first
      |> OgpEuskadi.Repo.one(prefix: "ogpe_test_tenant")

    %OgpEuskadi.Monitoring.Indicator{
      title_es: title_es,
      indicator_kind: indicator_kind,
      current_value: Decimal.new(1)
    }
  end


  # Activity can not be added without act as we need act.indicators to build activity_indicators
  # Example: tenant_insert(:act) |> add_activity |> tenant_insert()
  def add_activity(%OgpEuskadi.Plan.Act{} = act) do
    title_es = sequence(:title_es, &"Actividad #{&1}")
    %OgpEuskadi.Monitoring.Activity{
      act: act,
      title_es: title_es,
      description_es: "Descripción de la actividad #{title_es}",
      date: Date.utc_today,
      draft: true,
      activity_indicators: act.indicators |> Enum.map(fn ind -> build(:activity_indicator, indicator: ind) end )
    }
  end

  # NOTE: Always pass activity and indicator as attributes when building activity_indicators
  # Example: build(:activity_indicator, activity: activity, indicator: indicator)
  def activity_indicator_factory do
    %OgpEuskadi.Monitoring.ActivityIndicator{
      indicator_value: Decimal.new(1)
    }
  end

  def subdivision_factory do
    name_es = sequence(:name_es, &"Ámbito geográfico #{&1}")
    %OgpEuskadi.Config.Subdivision{
      name_es: name_es,
      description_es: "Descripción del #{name_es}"
    }
  end

  def api_key_factory do
    %OgpEuskadi.Auth.ApiKey{
      client: "Obaba",
      key: "5B17F8C1DA0A52620B64"
    }
  end

  def api_client_with_entity_factory do
    struct!(
      api_key_factory(),
      %{
        entity: build(:entity)
      }
    )
  end

end
